from openfisca_france.model.base import *

from numpy import round as round_, logical_or as or_, remainder as remainder_, datetime64


class ppa_eligibilite(Variable):
    value_type = bool
    entity = Famille
    label = 'Eligibilité à la PPA pour un mois'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        ppa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.ppa
        age_min = ppa.pa_cond.age_min
        condition_age_i = famille.members('age', period) >= age_min
        condition_age = famille.any(condition_age_i)

        return condition_age


class ppa_plancher_revenu_activite_etudiant(Variable):
    value_type = float
    entity = Individu
    label = "Plancher des revenus d'activité pour être éligible à la PPA en tant qu'étudiant"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        P = parameters(period)

        return (
            169
            * P.marche_travail.salaire_minimum.smic.smic_b_horaire
            * P.prestations_sociales.prestations_familiales.def_pac.revenu_plafond_pac_non_scolaire
            )


class ppa_eligibilite_etudiants(Variable):
    value_type = bool
    entity = Famille
    label = 'Eligibilité à la PPA (condition sur tout le trimestre)'
    reference = [
        'Article L842-1 du code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=46068A49B8592A593A05D64D8EDB045A.tplgfr26s_3?idArticle=LEGIARTI000031087527&cidTexte=LEGITEXT000006073189&dateTexte=20181226',
        'Article L842-2 du Code de la Sécurité Sociale',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=F2B88CEFCB83FCAFA4AA31671DAC89DD.tplgfr26s_3?idArticle=LEGIARTI000031087615&cidTexte=LEGITEXT000006073189&dateTexte=20181226'
        ]
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        ppa_majoree_eligibilite = famille('rsa_majore_eligibilite', period)

        etudiant_i = famille.members('etudiant', period)
        plancher_etudiant = famille.members('ppa_plancher_revenu_activite_etudiant', period)

        def condition_ressource(period2, plancher):
            revenu_activite = famille.members('ppa_revenu_activite_individu', period2)
            return plancher < revenu_activite

        m_1 = period.offset(-1, 'month')
        m_2 = period.offset(-2, 'month')
        m_3 = period.offset(-3, 'month')

        condition_etudiant_i = (
            condition_ressource(m_1, plancher_etudiant)
            * condition_ressource(m_2, plancher_etudiant)
            * condition_ressource(m_3, plancher_etudiant)
            )

        condition_non_etudiant_i = (
            not_(etudiant_i) * (
                condition_ressource(m_1, 0)
                + condition_ressource(m_2, 0)
                + condition_ressource(m_3, 0)
                )
            )

        condition_famille = famille.any(condition_non_etudiant_i + condition_etudiant_i, role = Famille.PARENT)
        return ppa_majoree_eligibilite + condition_famille


class ppa_montant_forfaitaire_familial_non_majore(Variable):
    value_type = float
    entity = Famille
    label = 'Montant forfaitaire familial (sans majoration)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        nb_parents = famille('nb_parents', period)
        nb_enfants = famille('rsa_nb_enfants', period)
        ppa_majoree_eligibilite = famille('rsa_majore_eligibilite', period)  # noqa F841
        ppa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.ppa

        nb_personnes = nb_parents + nb_enfants

        # Dans la formule "ppa_forfait_logement", le montant forfaitaire se calcule pour trois personnes dans le cas où le foyer se compose de trois personnes ou plus.
        taux_non_majore = (
            1
            + (nb_personnes >= 2) * ppa.pa_m.majoration_montant_maximal.couples_seul_avec_enfant
            + (nb_personnes >= 3) * ppa.pa_m.majoration_montant_maximal.couple_1_enfant_2e_enfant
            + (nb_personnes >= 4) * where(nb_parents == 1, ppa.pa_m.majoration_montant_maximal.par_enfant_supplementaire, ppa.pa_m.majoration_montant_maximal.couple_1_enfant_2e_enfant)
            # Si nb_parents == 1, pas de conjoint, la 4e personne est un enfant, donc le taux est de 40%.
            + max_(nb_personnes - 4, 0) * ppa.pa_m.majoration_montant_maximal.par_enfant_supplementaire
            )

        return ppa.pa_m.montant_de_base * taux_non_majore


class ppa_montant_forfaitaire_familial_majore(Variable):
    value_type = float
    entity = Famille
    label = 'Montant forfaitaire familial (avec majoration)'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        nb_enfants = famille('rsa_nb_enfants', period)
        ppa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.ppa

        taux_majore = (
            ppa.pa_m.majoration_isolement.femmes_enceintes
            + ppa.pa_m.majoration_isolement.par_enfant_charge
            * nb_enfants
            )

        return ppa.pa_m.montant_de_base * taux_majore


class ppa_revenu_activite(Variable):
    value_type = float
    entity = Famille
    label = "Revenu d'activité pris en compte pour la PPA"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        ppa_revenu_activite_i = famille.members(
            'ppa_revenu_activite_individu', period)
        ppa_revenu_activite = famille.sum(ppa_revenu_activite_i)

        return ppa_revenu_activite


class ppa_revenu_activite_individu(Variable):
    value_type = float
    entity = Individu
    label = "Revenu d'activité pris en compte pour la PPA (Individu) pour un mois"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article L842-4 du code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=358C45A1DF4FA63CC63BEC9456F63F18.tplgfr21s_3?idArticle=LEGIARTI000033813782&cidTexte=LEGITEXT000006073189',
        'Article R844-1 du code de la sécurité sociale',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=358C45A1DF4FA63CC63BEC9456F63F18.tplgfr21s_3?idArticle=LEGIARTI000031675756&cidTexte=LEGITEXT000006073189'
        ]

    def formula(individu, period, parameters):
        P = parameters(period)
        smic_horaire = P.marche_travail.salaire_minimum.smic.smic_b_horaire

        ressources = [
            'salaire_net',
            'revenus_stage_formation_pro',
            'remuneration_apprenti',
            'bourse_recherche',
            'indemnites_chomage_partiel',
            'rpns_auto_entrepreneur_benefice',
            'rsa_indemnites_journalieres_activite'
            ]

        revenus_mensualises = sum(individu(ressource, period) for ressource in ressources)

        revenus_tns_annualises = individu('ppa_rsa_derniers_revenus_tns_annuels_connus', period.this_year)

        revenus_activites = revenus_mensualises + revenus_tns_annualises

        # L'aah est pris en compte comme revenu d'activité si revenu d'activité hors aah > 29 * smic horaire brut
        seuil_aah_activite = P.prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_cond.seuil_aah_activite * smic_horaire
        aah_activite = (revenus_activites >= seuil_aah_activite) * individu('aah', period)

        return revenus_activites + aah_activite


class ppa_rsa_derniers_revenus_tns_annuels_connus(Variable):
    value_type = float
    entity = Individu
    label = 'Derniers revenus non salariés annualisés connus'
    definition_period = YEAR

    def formula(individu, period):

        def get_last_known(variable_name):
            valeur_n = individu(variable_name, period)
            valeur_n_1 = individu(variable_name, period.last_year)
            valeur_n_2 = individu(variable_name, period.n_2)
            return select(
                [valeur_n > 0, valeur_n_1 > 0, valeur_n_2 > 0],
                [valeur_n, valeur_n_1, valeur_n_2]
                ) / 12.

        return (
            get_last_known('rpns_benefice_exploitant_agricole')
            + get_last_known('rpns_autres_revenus')
            + get_last_known('rpns_micro_entreprise_benefice')
            )


class ppa_ressources_hors_activite(Variable):
    value_type = float
    entity = Famille
    label = 'Revenu hors activité pris en compte pour la PPA'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        aspa = famille('aspa', period)
        pf = famille('ppa_base_ressources_prestations_familiales', period)

        ass_i = famille.members('ass', period)
        ressources_hors_activite_i = famille.members('ppa_ressources_hors_activite_individu', period)

        return aspa + pf + famille.sum(ass_i + ressources_hors_activite_i)


class ppa_ressources_hors_activite_individu(Variable):
    value_type = float
    entity = Individu
    label = 'Revenu hors activité pris en compte pour la PPA (Individu) pour un mois'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        # Article L842-4 du code de la sécurité sociale
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=B1D8827D50F7B3CC603BB7D398E71AA8.tplgfr28s_3?idArticle=LEGIARTI000033813782&cidTexte=LEGITEXT000006073189&dateTexte=20181226',
        # Article R843-1 du code de la sécurité sociale
        'https://www.legifrance.gouv.fr/affichCode.do;jsessionid=3D8AB2FEC931285820291B1F952160BA.tpdila22v_2?idSectionTA=LEGISCTA000031694323&cidTexte=LEGITEXT000006073189&dateTexte=20160215'
        ]

    def formula(individu, period, parameters):
        P = parameters(period)
        smic_horaire = P.marche_travail.salaire_minimum.smic.smic_b_horaire

        def ressources_percues_au_cours_du_mois_considere():
            ressources = [
                'asi',
                'caah',
                'chomage_net',
                'retraite_nette',
                'retraite_combattant',
                'pensions_invalidite',
                'pensions_alimentaires_percues',
                'prestation_compensatoire',
                'prime_forfaitaire_mensuelle_reprise_activite',
                'rsa_indemnites_journalieres_hors_activite',
                ]

            return sum(individu(ressource, period) for ressource in ressources)

        def ressources_percues_il_y_a_deux_ans():
            ressources_individuelles_mensuelles = [
                'revenus_capital',
                'revenus_locatifs',
                ]

            revenus_annuels = sum(individu(ressource, period.offset(-2, 'year').this_year, options = [ADD]) for ressource in ressources_individuelles_mensuelles)
            plus_values_annuelles = (
                individu.foyer_fiscal('assiette_csg_plus_values', period.offset(-2, 'year').this_year)
                + individu.foyer_fiscal('rente_viagere_titre_onereux', period.offset(-2, 'year').this_year, options = [ADD])
                ) * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)

            return (revenus_annuels + plus_values_annuelles) / 12

        ressources_hors_activite_mensuel_i = (
            + ressources_percues_au_cours_du_mois_considere()
            + ressources_percues_il_y_a_deux_ans()
            )

        revenus_activites = individu('ppa_revenu_activite_individu', period)

        # L'AAH est prise en compte comme revenu d'activité si revenu d'activité hors aah > 29 * smic horaire brut
        seuil_aah_activite = P.prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_cond.seuil_aah_activite * smic_horaire
        aah_hors_activite = (revenus_activites < seuil_aah_activite) * individu('aah', period)

        return ressources_hors_activite_mensuel_i + aah_hors_activite


class ppa_base_ressources_prestations_familiales(Variable):
    value_type = float
    entity = Famille
    label = 'Prestations familiales prises en compte dans le calcul de la PPA'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        "Pour la prise en compte du complément familial, II. de l'article R844-4 du code de la sécurité sociale",
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=210D97A377874C24466BA7DE746FFF78.tplgfr27s_3?idArticle=LEGIARTI000031676000&cidTexte=LEGITEXT000006073189',
        "Pour la prise en compte des allocations familiales, 3° de l'article R844-5 du code de la sécurité sociale",
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=210D97A377874C24466BA7DE746FFF78.tplgfr27s_3?idArticle=LEGIARTI000031676016&cidTexte=LEGITEXT000006073189'
        ]

    def formula(famille, period, parameters):
        prestations = [
            'paje_base',
            'paje_clca',
            'paje_prepare',
            'paje_colca',
            'rsa_forfait_asf'
            ]

        result = sum(famille(prestation, period) for prestation in prestations)

        cf_non_majore_avant_cumul = famille('cf_non_majore_avant_cumul', period)
        cf = famille('cf', period)
        cf_pris_en_compte = (cf > 0) * cf_non_majore_avant_cumul

        af_base = famille('af_base', period)
        af = famille('af', period)
        af_prises_en_compte = min_(af_base, af)

        result = result + cf_pris_en_compte + af_prises_en_compte

        return result


class ppa_base_ressources(Variable):
    value_type = float
    entity = Famille
    label = 'Bases ressource prise en compte pour la PPA'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        ppa_revenu_activite = famille('ppa_revenu_activite', period)
        ppa_ressources_hors_activite = famille('ppa_ressources_hors_activite', period)
        return ppa_revenu_activite + ppa_ressources_hors_activite


class ppa_bonification(Variable):
    value_type = float
    entity = Individu
    label = 'Bonification de la PPA pour un individu'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        P = parameters(period)
        smic_horaire = P.marche_travail.salaire_minimum.smic.smic_b_horaire
        ppa_base = P.prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_m.montant_de_base
        revenu_activite = individu('ppa_revenu_activite_individu', period)
        seuil_1 = P.prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_m.bonification.seuil_bonification * smic_horaire
        seuil_2 = P.prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_m.bonification.seuil_max_bonification * smic_horaire
        bonification_max = round_(P.prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_m.bonification.taux_bonification_max * ppa_base, 2)
        bonification = bonification_max * (revenu_activite - seuil_1) / (seuil_2 - seuil_1)
        bonification = max_(bonification, 0)
        bonification = min_(bonification, bonification_max)

        return bonification


class ppa_forfait_logement(Variable):
    value_type = float
    entity = Famille
    label = "Forfait logement intervenant dans le calcul de la Prime pour l'Activité"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=9A3FFF4142B563EB5510DDE9F2870BF4.tplgfr41s_2?idArticle=LEGIARTI000031675988&cidTexte=LEGITEXT000006073189'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        np_pers = famille('nb_parents', period) + famille('rsa_nb_enfants', period)
        aide_logement = famille('aide_logement', period)
        statut_occupation_logement = famille.demandeur.menage('statut_occupation_logement', period)
        participation_frais = famille.demandeur.menage('participation_frais', period)
        loyer = famille.demandeur.menage('loyer', period)

        avantage_nature = or_(
            ((statut_occupation_logement == TypesStatutOccupationLogement.primo_accedant) + (statut_occupation_logement == TypesStatutOccupationLogement.proprietaire)) * not_(loyer),
            (statut_occupation_logement == TypesStatutOccupationLogement.loge_gratuitement) * not_(participation_frais)
            )

        avantage_al = aide_logement > 0

        params = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rsa
        ppa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.ppa

        # Le montant forfaitaire se calcule de la même manière que celle de la formule 'ppa_montant_forfaitaire_familial_non_majore',
        # sauf dans le cas où le foyer se compose de trois personnes ou plus, où le montant forfaitaire se calcule pour trois personnes seulement.
        taux_non_majore = (
            1
            + (np_pers >= 2) * ppa.pa_m.majoration_montant_maximal.couples_seul_avec_enfant
            + (np_pers >= 3) * ppa.pa_m.majoration_montant_maximal.couple_1_enfant_2e_enfant
            )

        montant_base = ppa.pa_m.montant_de_base * taux_non_majore

        montant_forfait = montant_base * (
            (np_pers == 1) * params.rsa_fl.forfait_logement.taux_1_personne
            + (np_pers == 2) * params.rsa_fl.forfait_logement.taux_2_personnes
            + (np_pers >= 3) * params.rsa_fl.forfait_logement.taux_3_personnes_ou_plus
            )

        montant_al = avantage_al * min_(aide_logement, montant_forfait)
        montant_nature = avantage_nature * montant_forfait

        return max_(montant_al, montant_nature)


class ppa_fictive_ressource_activite(Variable):
    value_type = float
    entity = Famille
    label = "Proportion de ressources provenant de l'activité prise en compte pour la primie d'activité fictive"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        pente = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_m.majoration_ressources_revenus_activite
        ppa_revenu_activite = famille('ppa_revenu_activite', period)

        return pente * ppa_revenu_activite


class ppa_fictive_montant_forfaitaire(Variable):
    value_type = float
    entity = Famille
    label = "Montant forfaitaire de la Prime pour l'Activité fictive"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        ppa_majoree_eligibilite = famille('rsa_majore_eligibilite', period)
        mff_non_majore = famille('ppa_montant_forfaitaire_familial_non_majore', period)
        mff_majore = famille('ppa_montant_forfaitaire_familial_majore', period)

        return where(ppa_majoree_eligibilite, mff_majore, mff_non_majore)


class ppa_fictive(Variable):
    value_type = float
    entity = Famille
    label = "Prime pour l'activité fictive pour un mois"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        forfait_logement = famille('ppa_forfait_logement', period)
        elig = famille('ppa_eligibilite', period)
        montant_forfaitaire_familialise = famille('ppa_fictive_montant_forfaitaire', period)
        ppa_base_ressources = famille('ppa_base_ressources', period)
        ppa_fictive_ressource_activite = famille('ppa_fictive_ressource_activite', period)
        bonification_i = famille.members('ppa_bonification', period)
        bonification = famille.sum(bonification_i)

        ppa_montant_base = (
            montant_forfaitaire_familialise
            + bonification
            + ppa_fictive_ressource_activite
            - ppa_base_ressources
            - forfait_logement
            )

        ppa_deduction = (
            montant_forfaitaire_familialise
            - ppa_base_ressources
            - forfait_logement
            )

        ppa_fictive = ppa_montant_base - max_(ppa_deduction, 0)
        ppa_fictive = max_(ppa_fictive, 0)
        return elig * ppa_fictive


class ppa(Variable):
    value_type = float
    entity = Famille
    label = "Prime Pour l'Activité"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add
    # Prime pour l'Activité sur service-public.fr
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F2882'

    def formula_2016_01_01(famille, period, parameters):
        seuil_non_versement = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.ppa.pa_m.montant_minimum_verse
        # éligibilité étudiants

        ppa_eligibilite_etudiants = famille('ppa_eligibilite_etudiants', period)
        ppa = famille('ppa_fictive', period.last_3_months, options = [ADD]) / 3
        ppa = ppa * ppa_eligibilite_etudiants * (ppa >= seuil_non_versement)

        return ppa


class ppa_mois_demande(Variable):
    value_type = date
    entity = Famille
    definition_period = ETERNITY
    label = "Date de la demande de la prime pour l'activité"


class ppa_indice_du_mois_trimestre_reference(Variable):
    value_type = int
    entity = Famille
    label = "Nombre de mois par rapport au mois de du précédent recalcul de la Prime pour l'Activité"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        ppa_mois_demande = famille('ppa_mois_demande', period)
        nombre_mois = (datetime64(period.start).astype('datetime64[M]') - ppa_mois_demande.astype('datetime64[M]')).astype('int')
        return remainder_(nombre_mois, 3)


class ppa_versee(Variable):
    value_type = float
    entity = Famille
    label = "Prime pour l'activité versée en prenant en compte la date de la demande"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        remainder = famille('ppa_indice_du_mois_trimestre_reference', period)
        return (
            + famille('ppa', period) * (remainder == 0)
            + famille('ppa', period.last_month) * (remainder == 1)
            + famille('ppa', period.last_month.last_month) * (remainder == 2)
            )



from openfisca_france import FranceTaxBenefitSystem
from openfisca_france.scenarios import init_single_entity


def test_coefficient_proratisation_only_contract_periods_wide():
    tax_benefit_system = FranceTaxBenefitSystem()
    scenario = tax_benefit_system.new_scenario()
    init_single_entity(scenario, period='2017',  # wide: we simulate for the year
        parent1=dict(salaire_de_base={'2017-11': 2300},
        effectif_entreprise=1,
        code_postal_entreprise='75001',
        categorie_salarie='prive_non_cadre',
        contrat_de_travail_debut={2017: '2017-11-01'},
        contrat_de_travail_fin={2017: '2017-12-01'},
        allegement_fillon_mode_recouvrement='progressif'))
    simulation = scenario.new_simulation()
    assert simulation.calculate('coefficient_proratisation', '2017-11') == 1
    assert simulation.calculate('coefficient_proratisation', '2017-12') == 0
    assert simulation.calculate('coefficient_proratisation', '2017-10') == 0
    assert simulation.calculate_add('coefficient_proratisation', '2017') == 1


def test_coefficient_proratisation_only_contract_periods_narrow():
    tax_benefit_system = FranceTaxBenefitSystem()
    scenario = tax_benefit_system.new_scenario()
    init_single_entity(scenario, period='2017-11',  # narrow: we simulate for the month
        parent1=dict(salaire_de_base={'2017-11': 2300},
        effectif_entreprise=1,
        code_postal_entreprise='75001',
        categorie_salarie='prive_non_cadre',
        contrat_de_travail_debut={2017: '2017-11-01'},
        contrat_de_travail_fin={2017: '2017-12-01'},
        allegement_fillon_mode_recouvrement='progressif'))
    simulation = scenario.new_simulation()
    assert simulation.calculate('coefficient_proratisation', '2017-11') == 1
    assert simulation.calculate('coefficient_proratisation', '2017-12') == 0
    assert simulation.calculate('coefficient_proratisation', '2017-10') == 0
    assert simulation.calculate_add('coefficient_proratisation', '2017') == 1



#! /usr/bin/env python

from setuptools import setup, find_packages
from pathlib import Path

# Read the contents of our README file for PyPi
this_directory = Path(__file__).parent
long_description = (this_directory / 'README.md').read_text()

setup(
    name = 'OpenFisca-France',
    version = '147.2.2',
    author = 'OpenFisca Team',
    author_email = 'contact@openfisca.fr',
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Scientific/Engineering :: Information Analysis',
        ],
    description = 'French tax and benefit system for OpenFisca',
    keywords = 'benefit france microsimulation social tax',
    license = 'http://www.fsf.org/licensing/licenses/agpl-3.0.html',
    license_files = ('LICENSE.AGPL.txt',),
    url = 'https://github.com/openfisca/openfisca-france',
    long_description=long_description,
    long_description_content_type='text/markdown',

    data_files = [
        (
            'share/openfisca/openfisca-france',
            ['CHANGELOG.md', 'README.md'],
            ),
        ],
    extras_require = {
        'inversion_revenus': [
            'scipy >= 0.17',
            ],
        'de_net_a_brut': [
            'scipy >= 0.17',
            ],
        'taxipp': [
            'pandas >= 0.13',
            ],
        'dev': [
            'autopep8 ==1.5.7',
            'flake8 >= 4.0.0, < 5.0.0',
            'flake8-print >= 5.0.0, < 6.0.0',
            'flake8-quotes >= 3.3.1, < 6.0.0',
            'pytest >= 5.0.0, < 7.0.0',
            'scipy >= 0.17',  # Only used to test de_net_a_brut reform
            'requests >= 2.8',
            'yamllint >=1.11.1,<1.27'
            ],
        'casd-dev': [
            # Same as dev with packages not available at CASD removed
            'autopep8 >=1.3.2',
            'pytest >= 5.0.0, < 7.0.0',
            'requests >= 2.8',
            'scipy >= 0.17',  # Only used to test de_net_a_brut reform
            ]
        },
    include_package_data = True,  # Will read MANIFEST.in
    install_requires = [
        'OpenFisca-Core >=35.8.0,<36.0',
        ],
    packages = find_packages(exclude = [
        'openfisca_france.tests*',
        'openfisca_france.assets.taxe_habitation.source*',
        ]),
    )



from openfisca_france.model.base import Individu, Variable, MONTH, set_input_dispatch_by_period
from openfisca_france.model.prestations.education import TypesScolarite


class garantie_pret_etudiant_eligibilite(Variable):
    value_type = bool
    label = "Éligibilité à la garantie de l'État au prêt étudiant"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F986'

    def formula_2020_06_08(individu, period, parameters):
        majeur = individu('majeur', period)

        age = individu('age', period)
        condition_age = (age <= parameters(period).prestations_sociales.aides_jeunes.garantie_pret_etudiant.age_max)

        etudiant_du_superieur = individu('scolarite', period) == TypesScolarite.enseignement_superieur
        condition_nationalite = individu('garantie_pret_etudiant_condition_nationalite', period)

        return majeur * etudiant_du_superieur * condition_age * condition_nationalite


class garantie_pret_etudiant_condition_nationalite(Variable):
    value_type = bool
    label = "Remplissage de la condition de nationalité ou de résidence pour la garantie de l'État au prêt étudiant"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F986'

    def formula_2021_02_19(individu, period, parameters):
        nationalite_francaise = individu('nationalite', period) == b'FR'

        ressortissant_eee = individu('ressortissant_eee', period)
        residence_continue_annees = individu('residence_continue_annees', period)
        duree_min_residence_continue_annees = parameters(period).prestations_sociales.aides_jeunes.garantie_pret_etudiant.duree_min_residence_continue_annees
        eligibilite_eee = ressortissant_eee * residence_continue_annees >= duree_min_residence_continue_annees

        return nationalite_francaise + eligibilite_eee





def montant_csg_crds(base_avec_abattement = None, base_sans_abattement = None, indicatrice_taux_plein = None,
        indicatrice_taux_reduit = None, law_node = None, plafond_securite_sociale = None):
    assert law_node is not None
    assert plafond_securite_sociale is not None
    if base_sans_abattement is None:
        base_sans_abattement = 0
    if base_avec_abattement is None:
        base = base_sans_abattement
    else:
        base = base_avec_abattement - law_node.abattement.calc(
            base_avec_abattement,
            factor = plafond_securite_sociale,
            round_base_decimals = 2,
            ) + base_sans_abattement
    if indicatrice_taux_plein is None and indicatrice_taux_reduit is None:
        return -law_node.taux * base
    else:
        return - (law_node.taux_plein * indicatrice_taux_plein + law_node.taux_reduit * indicatrice_taux_reduit) * base



from openfisca_france.model.base import *


class base_ressources_apa(Variable):
    value_type = float
    label = "Ressources considérées dans le calcul de l'APA"
    entity = Individu
    definition_period = MONTH

    def formula_2002(individu, period):
        period = period.first_month
        revenu_fiscal_de_reference = individu.foyer_fiscal('rfr', period.n_2) / 12
        aide_logement_montant = individu.famille('aide_logement_montant', period)
        valeur_locative_immo_non_loue = individu('valeur_locative_immo_non_loue', period)
        valeur_locative_terrains_non_loues = individu('valeur_locative_terrains_non_loues', period)

        return revenu_fiscal_de_reference - (
            valeur_locative_immo_non_loue + valeur_locative_terrains_non_loues + aide_logement_montant
            )


class apa_domicile_participation(Variable):
    value_type = float
    label = "Participation du bénéficiaire de l'APA à domicile en euros"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2002(individu, period, parameters):
        # Les départements doivent appliquer la nouvelle formule
        # entre le 1er mars 2016 et le 28 février 2017
        base_ressources_apa = individu('base_ressources_apa', period)
        en_couple = individu.famille('en_couple', period)
        autonomie = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees
        seuil_inf = autonomie.apa_domicile.seuil_revenu_en_part_mtp.seuil_inferieur
        seuil_sup = autonomie.apa_domicile.seuil_revenu_en_part_mtp.seuil_superieur
        majoration_tierce_personne = autonomie.mtp
        taux_min_participation = autonomie.apa_domicile.taux_de_participation.minimum
        taux_max_participation = autonomie.apa_domicile.taux_de_participation.maximum

        proratisation_couple = (
            1
            + en_couple
            * (autonomie.apa_domicile.division_ressources_menage_couples - 1)
            )

        dependance_plan_aide_domicile_accepte = individu('dependance_plan_aide_domicile_accepte', period)
        base_ressources_apa_domicile = base_ressources_apa / proratisation_couple

        condition_ressources_domicile = [
            base_ressources_apa_domicile <= (seuil_inf * majoration_tierce_personne),
            ((seuil_inf * majoration_tierce_personne) < base_ressources_apa_domicile) * (base_ressources_apa_domicile <= (seuil_sup * majoration_tierce_personne)),
            base_ressources_apa_domicile > (seuil_sup * majoration_tierce_personne),
            ]

        taux_participation = [
            taux_min_participation,
            (base_ressources_apa_domicile - seuil_inf * majoration_tierce_personne) / ((seuil_sup - seuil_inf) * majoration_tierce_personne) * taux_max_participation,
            taux_max_participation,
            ]

        return select(condition_ressources_domicile, taux_participation) * dependance_plan_aide_domicile_accepte

    def formula_2016_03_01(individu, period, parameters):
        # Les départements doivent appliquer la nouvelle formule
        # entre le 1er mars 2016 et le 28 février 2017
        base_ressources_apa = individu('base_ressources_apa', period)
        en_couple = individu.famille('en_couple', period)
        dependance_plan_aide_domicile_accepte = individu('dependance_plan_aide_domicile_accepte', period)
        parameters = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees
        majoration_tierce_personne = parameters.mtp

        proratisation_couple = (
            1
            + en_couple
            * (parameters.apa_domicile.division_ressources_menage_couples - 1)
            )

        base_ressources_apa_domicile = base_ressources_apa / proratisation_couple

        premier_seuil = 0.317 * majoration_tierce_personne
        second_seuil = 0.498 * majoration_tierce_personne
        condlist = [
            dependance_plan_aide_domicile_accepte <= premier_seuil,
            (premier_seuil <= dependance_plan_aide_domicile_accepte) * (dependance_plan_aide_domicile_accepte <= second_seuil),
            dependance_plan_aide_domicile_accepte >= second_seuil,
            ]
        choicelist_1 = [
            dependance_plan_aide_domicile_accepte,
            premier_seuil,
            premier_seuil,
            ]
        choicelist_2 = [
            0,
            dependance_plan_aide_domicile_accepte - premier_seuil,
            second_seuil,
            ]
        choicelist_3 = [
            0,
            0,
            dependance_plan_aide_domicile_accepte - (premier_seuil + second_seuil)
            ]
        A_1 = select(condlist, choicelist_1)
        A_2 = select(condlist, choicelist_2)
        A_3 = select(condlist, choicelist_3)

        apa_domicile_participation = min_(
            0.9 * dependance_plan_aide_domicile_accepte,
            0.9
            * max_(0, base_ressources_apa_domicile - 0.725 * majoration_tierce_personne)
            / (1.945 * majoration_tierce_personne)
            * (
                A_1
                + A_2 * (
                    (1 - 0.4) * base_ressources_apa_domicile / (1.945 * majoration_tierce_personne)
                    + (0.4 * 2.67 * majoration_tierce_personne - 0.725 * majoration_tierce_personne) / (1.945 * majoration_tierce_personne)
                    )
                + A_3 * (
                    (1 - 0.2) * base_ressources_apa_domicile / (1.945 * majoration_tierce_personne)
                    + (0.2 * 2.67 * majoration_tierce_personne - 0.725 * majoration_tierce_personne) / (1.945 * majoration_tierce_personne)
                    )
                )
            )

        return apa_domicile_participation


class apa_eligibilite(Variable):
    value_type = bool
    entity = Individu
    label = "Allocation personalisée d'autonomie - Éligibilité"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2002(individu, period, parameters):
        period = period.start.offset('first-of', 'month').period('month')
        parameters = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees
        age = individu('age', period)
        apa_age_min = parameters.apa_domicile.condition_age

        gir = individu('gir', period)
        eligibilite_gir = (
            (gir == TypesGir.gir_1)
            + (gir == TypesGir.gir_2)
            + (gir == TypesGir.gir_3)
            + (gir == TypesGir.gir_4)
            )

        return (age >= apa_age_min) * eligibilite_gir


class apa_domicile_taux_participation(Variable):
    value_type = float
    label = "Taux de participation du bénéficiaire à l'APA à domicile"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula_2002(individu, period, parameters):
        apa_domicile = individu('apa_domicile', period)
        dependance_plan_aide_domicile_accepte = individu('dependance_plan_aide_domicile_accepte', period)
        apa_domicile_participation = individu('apa_domicile_participation', period)

        return (apa_domicile >= 0) * apa_domicile_participation / dependance_plan_aide_domicile_accepte


class apa_domicile(Variable):
    value_type = float
    label = "Allocation personalisée d'autonomie"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2002(individu, period, parameters):
        period = period.start.offset('first-of', 'month').period('month')
        apa_domicile = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees.apa_domicile
        apa_eligibilite = individu('apa_eligibilite', period)
        smic_brut_horaire = parameters(period).marche_travail.salaire_minimum.smic.smic_b_horaire
        seuil_non_versement = apa_domicile.seuil_versement_en_part_smic_brut_horaire * smic_brut_horaire
        dependance_plan_aide_domicile_accepte = individu('dependance_plan_aide_domicile_accepte', period)

        apa_domicile_participation = individu('apa_domicile_participation', period)

        apa = dependance_plan_aide_domicile_accepte - apa_domicile_participation
        return apa * (apa >= seuil_non_versement) * apa_eligibilite


class apa_etablissement(Variable):
    value_type = float
    label = "Allocation personalisée d'autonomie en institution"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2002(individu, period, parameters):
        period = period.start.offset('first-of', 'month').period('month')
        perte_autonomie_personnes_agees = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees
        smic_brut_horaire = parameters(period).marche_travail.salaire_minimum.smic.smic_b_horaire
        seuil_non_versement = perte_autonomie_personnes_agees.apa_institution.seuil_versement_en_part_smic_brut_horaire * smic_brut_horaire
        en_couple = individu.famille('en_couple', period)
        apa_eligibilite = individu('apa_eligibilite', period)
        gir = individu('gir', period)  # noqa F841
        base_ressources_apa = individu('base_ressources_apa', period)

        proratisation_couple_etablissement = (
            1
            + en_couple
            * (perte_autonomie_personnes_agees.apa_institution.division_ressources_menage_couples - 1)
            )

        base_ressources_apa_etablissement = base_ressources_apa / proratisation_couple_etablissement
        dependance_tarif_etablissement_gir_5_6 = individu('dependance_tarif_etablissement_gir_5_6', period)
        dependance_tarif_etablissement_gir_dependant = individu('dependance_tarif_etablissement_gir_dependant', period)
        seuil_inf_inst = perte_autonomie_personnes_agees.apa_institution.seuil_inferieur
        seuil_sup_inst = perte_autonomie_personnes_agees.apa_institution.seuil_superieur
        majoration_tierce_personne = perte_autonomie_personnes_agees.mtp

        conditions_ressources = [
            base_ressources_apa_etablissement <= seuil_inf_inst * majoration_tierce_personne,
            (seuil_inf_inst * majoration_tierce_personne < base_ressources_apa_etablissement) * (base_ressources_apa_etablissement <= seuil_sup_inst * majoration_tierce_personne),
            base_ressources_apa > seuil_sup_inst * majoration_tierce_personne
            ]

        participations = [
            dependance_tarif_etablissement_gir_5_6,
            (
                dependance_tarif_etablissement_gir_5_6
                + (dependance_tarif_etablissement_gir_dependant - dependance_tarif_etablissement_gir_5_6)
                * (
                    (base_ressources_apa - seuil_inf_inst * majoration_tierce_personne)
                    / ((seuil_sup_inst - seuil_inf_inst) * majoration_tierce_personne)
                    * 0.80
                    )
                ),
            dependance_tarif_etablissement_gir_5_6
            + (dependance_tarif_etablissement_gir_dependant - dependance_tarif_etablissement_gir_5_6)
            * 0.80
            ]

        participation_beneficiaire = select(conditions_ressources, participations)
        taux_reste_a_vivre = perte_autonomie_personnes_agees.apa_institution.taux_reste_a_vivre
        participation_beneficiaire = min_(
            participation_beneficiaire,
            max_(base_ressources_apa_etablissement * (1 - taux_reste_a_vivre), 0)
            )
        apa = dependance_tarif_etablissement_gir_dependant - participation_beneficiaire

        eligibilite_etablissement = (
            (dependance_tarif_etablissement_gir_5_6 > 0) * (dependance_tarif_etablissement_gir_dependant > 0)
            )  # permet de sélectionner les individus vivant en établissement éligible.

        return apa * (apa >= seuil_non_versement) * eligibilite_etablissement * apa_eligibilite


class TypesGir(Enum):
    __order__ = 'non_defini gir_1 gir_2 gir_3 gir_4 gir_5 gir_6'  # Needed to preserve the enum order in Python 2
    non_defini = 'Non défini'
    gir_1 = 'Gir 1'
    gir_2 = 'Gir 2'
    gir_3 = 'Gir 3'
    gir_4 = 'Gir 4'
    gir_5 = 'Gir 5'
    gir_6 = 'Gir 6'


class gir(Variable):
    value_type = Enum
    possible_values = TypesGir
    default_value = TypesGir.non_defini
    entity = Individu
    label = "Groupe iso-ressources de l'individu"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class dependance_plan_aide_domicile(Variable):
    value_type = float
    entity = Individu
    label = "Coût du plan d'aide à domicile pour une personne dépendante"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class dependance_tarif_etablissement_gir_1_2(Variable):
    value_type = float
    entity = Individu
    label = "Tarif dépendance de l'établissement pour les GIR 1 et 2"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class dependance_tarif_etablissement_gir_3_4(Variable):
    value_type = float
    entity = Individu
    label = "Tarif dépendance de l'établissement pour les GIR 3 et 4"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class dependance_tarif_etablissement_gir_5_6(Variable):
    value_type = float
    entity = Individu
    label = "Tarif dépendance de l'établissement pour les GIR 5 et 6"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class dependance_tarif_etablissement_gir_dependant(Variable):
    value_type = float
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    label = "Tarif dépendance de l'établissement pour le GIR de la personne dépendante"
    reference = ['https://www.service-public.fr/particuliers/vosdroits/F10009']

    def formula_2002(individu, period):
        gir = individu('gir', period)
        tarif_gir_1_2 = individu('dependance_tarif_etablissement_gir_1_2', period)
        tarif_gir_3_4 = individu('dependance_tarif_etablissement_gir_3_4', period)

        # Vérifie si l'individu est rattaché à l'un des groupes 1 à 4 de la grille Aggir
        gir_1_2 = (gir == TypesGir.gir_1) + (gir == TypesGir.gir_2)
        gir_3_4 = (gir == TypesGir.gir_3) + (gir == TypesGir.gir_4)

        # Sélectionne le tarif correspondant à la grille de la personne
        return select([gir_1_2, gir_3_4], [tarif_gir_1_2, tarif_gir_3_4])


class apa_urgence_domicile(Variable):
    value_type = float
    label = "Allocation personalisée d'autonomie d'urgence à domicile"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2002(individu, period, parameters):
        period = period.first_month
        autonomie = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees
        majoration_tierce_personne = autonomie.mtp
        plafond_gir1 = autonomie.apa_domicile.plafond_apa_domicile_en_part_mtp.gir_1
        part_urgence_domicile = autonomie.apa_domicile.part_plafond_apa_domicile
        return part_urgence_domicile * plafond_gir1 * majoration_tierce_personne


class apa_urgence_institution(Variable):
    value_type = float
    label = "Allocation personalisée d'autonomie en institution"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2002(individu, period, parameters):
        period = period.start.offset('first-of', 'month').period('month')
        dependance_tarif_etablissement_gir_1_2 = individu('dependance_tarif_etablissement_gir_1_2', period)
        part_urgence_institution = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees.apa_institution.part_tarif_dependance
        apa_urgence_institution = part_urgence_institution * dependance_tarif_etablissement_gir_1_2
        return apa_urgence_institution


class dependance_plan_aide_domicile_accepte(Variable):
    value_type = float
    label = "Coût du plan d'aide plafonné pris en compte pour la détermination de l'APA"
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        # Code de l'action sociale et des familles - Article R232-10-1
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=4D213136F764CDAC77B33F705B4DE178.tplgfr41s_1?idArticle=LEGIARTI000032133764&cidTexte=LEGITEXT000006074069&dateTexte=20170929&categorieLien=id&oldAction=&nbResultRech='
        ]

    def formula_2002(individu, period, parameters):
        gir = individu('gir', period)
        dependance_plan_aide_domicile = individu('dependance_plan_aide_domicile', period)
        parameters_autonomie = parameters(period).prestations_sociales.prestations_etat_de_sante.perte_autonomie_personnes_agees

        plafond_gir1 = parameters_autonomie.apa_domicile.plafond_apa_domicile_en_part_mtp.gir_1
        plafond_gir2 = parameters_autonomie.apa_domicile.plafond_apa_domicile_en_part_mtp.gir_2
        plafond_gir3 = parameters_autonomie.apa_domicile.plafond_apa_domicile_en_part_mtp.gir_3
        plafond_gir4 = parameters_autonomie.apa_domicile.plafond_apa_domicile_en_part_mtp.gir_4
        majoration_tierce_personne = parameters_autonomie.mtp

        condition_plafond_par_gir = [
            gir == TypesGir.gir_1,
            gir == TypesGir.gir_2,
            gir == TypesGir.gir_3,
            gir == TypesGir.gir_4,
            ]
        valeur_plafond_par_gir = [
            plafond_gir1 * majoration_tierce_personne,
            plafond_gir2 * majoration_tierce_personne,
            plafond_gir3 * majoration_tierce_personne,
            plafond_gir4 * majoration_tierce_personne,
            ]
        plafond_par_gir = select(condition_plafond_par_gir, valeur_plafond_par_gir)
        return min_(plafond_par_gir, dependance_plan_aide_domicile)



#! /usr/bin/env python
'''
Unfold parameters located in path
'''


import argparse
import glob
import logging
import os
import re
import sys

import yaml


DATE_REGEXP = re.compile(
    r'^(0001-01-01|[12]\d{3}-(0[1-9]|1[012])-(0[1-9]|[12]\d|3[01]))$'
    )
SHARED_KEYS = ['description', 'documentation', 'file_path', 'metadata', 'name', 'unit']


app_name = os.path.splitext(os.path.basename(__file__))[0]
logger = logging.getLogger(app_name)


# See https://ttl255.com/yaml-anchors-and-aliases-and-how-to-disable-them/
class NoAliasDumper(yaml.SafeDumper):
    def ignore_aliases(self, data):
        return True


def add_dated_metadata_to_parameter(
        parameter,
        dates,
        note_by_date,
        reference_by_date,
        official_journal_date_by_date,
        encountered_dates,
        ):
    metadata = parameter.get('metadata', {})
    for date in dates:
        for key, value_by_date in (
                ('notes', note_by_date),
                ('official_journal_date', official_journal_date_by_date),
                ('reference', reference_by_date),
                ):
            value_at_date = value_by_date.get(date)
            if value_at_date is not None:
                metadata_value = metadata.get(key)
                if metadata_value is None:
                    metadata_value = metadata[key] = {}
                if metadata_value.get(date) is None:
                    metadata_value[date] = value_at_date
    if metadata and parameter.get('metadata') is None:
        parameter['metadata'] = metadata
    encountered_dates.update(dates)


def unfold_file(yaml_source_file_path, remove_source = False):
    assert yaml_source_file_path.endswith('.yaml'), '{} must be a YAML file path.'.format(yaml_source_file_path)
    with open(yaml_source_file_path, 'r', encoding='utf-8') as yaml_source_file:
        source_parameter = yaml.safe_load(yaml_source_file)
    target_dir, yaml_source_filename = os.path.split(yaml_source_file_path)
    id = os.path.splitext(yaml_source_filename)[0]
    parameter_is_unfolded = unfold_parameter([id], source_parameter, target_dir, {}, {}, {}, set())
    if remove_source and parameter_is_unfolded:
        os.remove(yaml_source_file_path)


def unfold_path(yaml_source_path, remove_source = False):
    if os.path.isfile(yaml_source_path):
        logger.debug('Unfolding file {}'.format(yaml_source_path))
        unfold_file(yaml_source_file_path = yaml_source_path, remove_source = remove_source)
    else:
        assert os.path.isdir(yaml_source_path), '{} is not a directory'.format(yaml_source_path)
        if len(os.listdir(yaml_source_path)) == 0:
            return
        logger.debug('Unfolding directory {}'.format(yaml_source_path))

        yaml_source_files_paths = glob.glob(os.path.join(yaml_source_path, '**/*.yaml'), recursive = True)
        for yaml_source_file_path in yaml_source_files_paths:
            if yaml_source_file_path.endswith('index.yaml'):
                continue
            unfold_path(yaml_source_file_path, remove_source = remove_source)


def unfold_parameter(
        ids,
        source_parameter,
        target_dir,
        # Metadata that must dispatched to children
        # when they have a value with a matching date.
        note_by_date,
        reference_by_date,
        official_journal_date_by_date,
        # Filled with dates appearing in children.
        encountered_dates,
        ):
    assert 'reference' not in source_parameter, \
        f"Property 'reference' of parameter {'.'.join(ids)} must be moved into metadata"
    if 'brackets' in source_parameter:
        # Parameter is a scale.
        brackets = source_parameter['brackets']
        dates = set()
        for key in ('amount', 'average_rate', 'base', 'rate', 'threshold'):
            value_by_date = brackets.get(key)
            if value_by_date is not None:
                dates.update(value_by_date.keys())
        add_dated_metadata_to_parameter(
            source_parameter,
            dates,
            note_by_date,
            reference_by_date,
            official_journal_date_by_date,
            encountered_dates,
            )
        write_parameter(ids, source_parameter, target_dir)
        return False  # File has not been unfolded.
    else:
        values = source_parameter.get('values')
        if values is None:
            children_keys = [
                key
                for key in source_parameter
                if key not in SHARED_KEYS
                ]
            assert children_keys
            if all(re.match(DATE_REGEXP, child_key) for child_key in children_keys):
                # Parameter is a value.
                add_dated_metadata_to_parameter(
                    source_parameter,
                    children_keys,
                    note_by_date,
                    reference_by_date,
                    official_journal_date_by_date,
                    encountered_dates,
                    )
                write_parameter(ids, source_parameter, target_dir)
                return False  # File has not been unfolded.
            else:
                # Parameter is a node to unfold.

                metadata = source_parameter.get('metadata', {})
                children_note_by_date = {**note_by_date, **metadata.get('notes', {})}
                children_reference_by_date = {
                    **reference_by_date,
                    **metadata.get('reference', {}),
                    }
                children_official_journal_date_by_date = {
                    **official_journal_date_by_date,
                    **metadata.get('official_journal_date', {}),
                    }
                for child_id in children_keys:
                    unfold_parameter(
                        ids + [child_id],
                        source_parameter[child_id],
                        target_dir,
                        children_note_by_date,
                        children_reference_by_date,
                        children_official_journal_date_by_date,
                        encountered_dates,
                        )

                # Remove metadata dispatched to children from index metadata.
                for key in ('notes', 'reference', 'official_journal_date'):
                    value_by_date = metadata.get(key)
                    if value_by_date is not None:
                        for date in encountered_dates:
                            value_by_date.pop(date, None)
                        if not value_by_date:
                            del metadata[key]
                if not metadata:
                    source_parameter.pop('metadata', None)

                index = {
                    key: value
                    for key, value in source_parameter.items()
                    if key in SHARED_KEYS
                    }
                if index:
                    write_parameter(ids + ['index'], index, target_dir)
                return True  # File has been unfolded.
        else:
            # Parameter is a value.
            add_dated_metadata_to_parameter(
                source_parameter,
                source_parameter['values'].keys(),
                note_by_date,
                reference_by_date,
                official_journal_date_by_date,
                encountered_dates,
                )
            write_parameter(ids, source_parameter, target_dir)
            return False  # File has not been unfolded.


def write_parameter(ids, parameter, target_dir):
    dir = os.path.join(target_dir, *ids[:-1])
    os.makedirs(dir, exist_ok=True)
    with open(os.path.join(dir, f'{ids[-1]}.yaml'), 'w', encoding='utf-8') as parameter_file:
        yaml.dump(parameter, parameter_file, allow_unicode=True, Dumper=NoAliasDumper)


def main():
    parser = argparse.ArgumentParser(description = __doc__)
    parser.add_argument('-p', '--path', help = 'Parameters path a file or a directory)')
    parser.add_argument('-r', '--remove-source', action = 'store_true', default = False, help = 'remove source files')
    parser.add_argument('-v', '--verbose', action = 'store_true', default = False, help = 'increase output verbosity')
    args = parser.parse_args()
    logging.basicConfig(level = logging.DEBUG if args.verbose else logging.WARNING)
    yaml_source_path = args.path
    remove_source = args.remove_source
    unfold_path(yaml_source_path, remove_source = remove_source)


if __name__ == '__main__':
    sys.exit(main())



from openfisca_france.model.base import *

from openfisca_france.model.prestations.education import TypesScolarite


class seances_sante_psy_etudiant(Variable):
    value_type = int
    entity = Individu
    label = "Nombre de séances prises en charge par l'Aide Santé Psy Étudiant"
    definition_period = MONTH
    reference = 'https://www.service-public.fr/particuliers/actualites/A14726'
    set_input = set_input_divide_by_period

    def formula_2021_03_10(individu, period, parameters):
        '''
        Le nombre maximum de séances est donné sans prendre en compte les passages obligatoires intermédiaires chez un médecin.
        '''
        etudiant = individu('etudiant', period)
        enseignement_superieur = individu('scolarite', period) == TypesScolarite.enseignement_superieur

        return etudiant * enseignement_superieur * parameters(period).prestations_sociales.aides_jeunes.sante_psy.etudiant.seances_max



import logging
from numpy import around
from openfisca_france.model.base import *


log = logging.getLogger(__name__)


class reductions_deplafonnees(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Réductions d'impôt sur le revenu déplafonnées"
    definition_period = YEAR

    def formula_2013_01_01(foyer_fiscal, period, parameters):

        reductions_sans_plafond = [
            'accueil_dans_etablissement_personnes_agees',
            'dfppce',
            'frais_de_comptabilite',
            'assvie',
            'reduction_cotisations_syndicales',
            'creaen',
            'interets_paiements_differes_agriculteurs',
            'mecena',
            'prestations_compensatoires',
            'interets_emprunt_reprise_societe',
            'restauration_patrimoine_bati',  # Malraux, non plafonnées pour les investissements réalisés après 2013
            'reduction_enfants_scolarises',
            'accult',
            'rente_survie',
            ]

        # Step 4: Get other uncapped reductions
        red_deplaf = sum([around(foyer_fiscal(reduction, period)) for reduction in reductions_sans_plafond])

        return red_deplaf


class accult(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Acquisition de biens culturels'
    definition_period = YEAR
    end = '2020-12-31'

    def formula_2002(foyer_fiscal, period, parameters):
        '''
        Acquisition de biens culturels (case 7UO)
        2002-
        '''
        f7uo = foyer_fiscal('f7uo', period)
        P = parameters(period).impot_revenu.credits_impots.accult

        return P.taux * f7uo


class frais_de_comptabilite(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'frais_de_comptabilite'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        Frais de comptabilité et d'adhésion à un CGA ou AA
        2002-
        '''
        f7ff = foyer_fiscal('f7ff', period)
        f7fg = foyer_fiscal('f7fg', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.frais_de_comptabilite

        return min_(f7ff, P.plafond * f7fg)


class assvie(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'assvie'
    end = '2004-12-31'
    definition_period = YEAR

    def formula_2002_01_01(foyer_fiscal, period, parameters):
        '''
        Assurance-vie (cases GW, GX et GY de la 2042)
        2002-2004
        '''
        nb_pac = foyer_fiscal('nb_pac', period)
        f7gw = foyer_fiscal('f7gw_2004', period)
        f7gx = foyer_fiscal('f7gx_2004', period)
        f7gy = foyer_fiscal('f7gy_2004', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.assurances_vie

        plafond = P.plafond + nb_pac * P.increment
        return P.taux * min_(f7gw + f7gx + f7gy, plafond)


class creaen(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'creaen'
    definition_period = YEAR
    end = '2014-12-31'

    def formula_2006_01_01(foyer_fiscal, period, parameters):
        '''
        Aide aux créateurs et repreneurs d'entreprises
        2006-2008
        '''
        f7fy = foyer_fiscal('f7fy_2011', period)
        f7gy = foyer_fiscal('f7gy_2010', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.aide_createurs_repreneurs_entreprise

        return (P.reduction * f7fy + P.surplus_si_invalide * f7gy)

    def formula_2009_01_01(foyer_fiscal, period, parameters):
        '''
        Aide aux créateurs et repreneurs d'entreprises
        2009
        '''
        f7fy = foyer_fiscal('f7fy_2011', period)
        f7gy = foyer_fiscal('f7gy_2010', period)
        f7jy = foyer_fiscal('f7jy_2010', period)
        f7hy = foyer_fiscal('f7hy_2011', period)
        f7ky = foyer_fiscal('f7ky_2011', period)
        f7iy = foyer_fiscal('f7iy_2011', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.aide_createurs_repreneurs_entreprise

        return (
            P.reduction * ((f7jy + f7fy) + f7hy / 2)
            + P.surplus_si_invalide * ((f7ky + f7gy) + f7iy / 2)
            )

    def formula_2010_01_01(foyer_fiscal, period, parameters):
        '''
        Aide aux créateurs et repreneurs d'entreprises
        2010-2011
        '''
        f7fy = foyer_fiscal('f7fy_2011', period)
        f7gy = foyer_fiscal('f7gy_2010', period)
        f7jy = foyer_fiscal('f7jy_2010', period)
        f7hy = foyer_fiscal('f7hy_2011', period)
        f7ky = foyer_fiscal('f7ky_2011', period)
        f7iy = foyer_fiscal('f7iy_2011', period)
        f7ly = foyer_fiscal('f7ly_2010', period)
        f7my = foyer_fiscal('f7my_2010', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.aide_createurs_repreneurs_entreprise

        return (
            P.reduction * ((f7jy + f7fy) + (f7hy + f7ly) / 2)
            + P.surplus_si_invalide * ((f7ky + f7gy) + (f7iy + f7my) / 2)
            )

    def formula_2012_01_01(foyer_fiscal, period, parameters):
        '''
        Aide aux créateurs et repreneurs d'entreprises
        2012-
        '''
        f7ly = foyer_fiscal('f7ly_2010', period)
        f7my = foyer_fiscal('f7my_2010', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.aide_createurs_repreneurs_entreprise

        return (
            P.reduction * (f7ly / 2)
            + P.surplus_si_invalide * (f7my / 2)
            )


class accueil_dans_etablissement_personnes_agees(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Hébergement santé'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        Dépenses d'accueil dans un établissement pour personnes âgées dépendantes
        ?-
        '''
        f7cd = foyer_fiscal('f7cd', period)
        f7ce = foyer_fiscal('f7ce', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.accueil_dans_etablissement_personnes_agees

        return P.taux * (min_(f7cd, P.plafond + min_(f7ce, P.plafond)))


class dfppce(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Dons à des organismes d'intérêt général et dons pour le financement des partis politiques"
    reference = 'http://bofip.impots.gouv.fr/bofip/5823-PGP'
    definition_period = YEAR

    def formula_2002_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons

        base = f7uf
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2004_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7xs = foyer_fiscal('f7xs', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons

        base = f7uf + f7xs
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2005_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons

        base = f7uf + f7xs + f7xt
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2006_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons

        base = f7uf + f7xs + f7xt + f7xu
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2007_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons

        base = f7uf + f7xs + f7xt + f7xu + f7xw
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2008_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        f7xy = foyer_fiscal('f7xy', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons

        base = f7uf + f7xs + f7xt + f7xu + f7xw + f7xy
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2011_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales (2011-2013)
        '''
        rni = foyer_fiscal('rni', period)
        f7ud = foyer_fiscal('f7ud', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        f7xy = foyer_fiscal('f7xy', period)
        f7va = foyer_fiscal('f7va', period)
        f7vc = foyer_fiscal('f7vc', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons
        plafond_reduction_don_coluche = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.plafond

        report_f7va_f7ud = max_(0, f7va + f7ud - plafond_reduction_don_coluche)
        base = f7uf + f7vc + f7xs + f7xt + f7xu + f7xw + f7xy + report_f7va_f7ud
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2012_01_01(foyer_fiscal, period, parameters):
        '''
        Dons aux autres oeuvres et dons effectués pour le financement des partis
        politiques et des campagnes électorales (2011-2013)
        '''
        rni = foyer_fiscal('rni', period)
        f7ud = foyer_fiscal('f7ud', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        f7xy = foyer_fiscal('f7xy', period)
        f7va = foyer_fiscal('f7va', period)
        f7vc = foyer_fiscal('f7vc', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.dons
        plafond_reduction_don_coluche = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.plafond

        report_f7va_f7ud = max_(0, f7va + f7ud - plafond_reduction_don_coluche)
        base = f7uf + f7vc + f7xs + f7xt + f7xu + f7xw + f7xy + report_f7va_f7ud
        max1 = P.dons_interet_general.plafond * rni
        return P.dons_aux_oeuvres.taux * min_(base, max1)

    def formula_2013_01_01(foyer_fiscal, period, parameters):
        '''
        Dons versés à d’autres organismes d’intérêt général,
        aux associations d’utilité publique, aux candidats aux élections (2019)
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7uh = foyer_fiscal('f7uh', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        f7xy = foyer_fiscal('f7xy', period)
        f7va = foyer_fiscal('f7va', period)
        f7ud = foyer_fiscal('f7ud', period)
        f7vc = foyer_fiscal('f7vc', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.dons
        plafond_reduction_don_coluche = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.plafond
        taux_donapd = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.taux
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)

        red_7ud_7va = min_(plafond_reduction_don_coluche, f7va + f7ud) * taux_donapd
        report_f7va_f7ud = max_(0, f7va + f7ud - plafond_reduction_don_coluche)

        dons_partipol = min_(P.dons_aux_partis_politiques.plafond_seul * (1 + maries_ou_pacses), f7uh)

        base = f7uf + f7vc + f7xs + f7xt + f7xu + f7xw + f7xy + report_f7va_f7ud + dons_partipol
        max = P.dons_interet_general.plafond * rni

        return red_7ud_7va + P.dons_aux_oeuvres.taux * min_(base, max)

    def formula_2019_01_01(foyer_fiscal, period, parameters):
        '''
        Dons versés à d’autres organismes d’intérêt général,
        aux associations d’utilité publique, aux candidats aux élections (2019)
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7uh = foyer_fiscal('f7uh', period)
        f7ue = foyer_fiscal('f7ue', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        f7xy = foyer_fiscal('f7xy', period)
        f7va = foyer_fiscal('f7va', period)
        f7ud = foyer_fiscal('f7ud', period)
        f7vc = foyer_fiscal('f7vc', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.dons
        PND = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_notre_dame
        plafond_reduction_don_coluche = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.plafond
        taux_donapd = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.taux
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)

        red_7ud_7va = min_(plafond_reduction_don_coluche, f7va + f7ud) * taux_donapd
        report_f7va_f7ud = max_(0, f7va + f7ud - plafond_reduction_don_coluche)

        red_notre_dame = min_(PND.plafond, f7ue) * PND.taux
        report_notre_dame = max_(0, f7ue - PND.plafond)

        dons_partipol = min_(P.dons_aux_partis_politiques.plafond_seul * (1 + maries_ou_pacses), f7uh)

        base = f7uf + f7vc + f7xs + f7xt + f7xu + f7xw + f7xy + report_f7va_f7ud + dons_partipol + report_notre_dame
        max = P.dons_interet_general.plafond * rni

        return red_notre_dame + red_7ud_7va + P.dons_aux_oeuvres.taux * min_(base, max)

    def formula_2020_01_01(foyer_fiscal, period, parameters):
        '''
        Dons versés à d’autres organismes d’intérêt général,
        aux associations d’utilité publique, aux candidats aux élections (2020)
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7uh = foyer_fiscal('f7uh', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        f7xy = foyer_fiscal('f7xy', period)
        f7va = foyer_fiscal('f7va', period)
        f7ud = foyer_fiscal('f7ud', period)
        f7vc = foyer_fiscal('f7vc', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.dons
        plafond_reduction_don_coluche = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.plafond
        taux_donapd = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.taux
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)

        red_7ud_7va = min_(plafond_reduction_don_coluche, f7va + f7ud) * taux_donapd
        report_f7va_f7ud = max_(0, f7va + f7ud - plafond_reduction_don_coluche)

        dons_partipol = min_(P.dons_aux_partis_politiques.plafond_seul * (1 + maries_ou_pacses), f7uh)

        base = f7uf + f7vc + f7xs + f7xt + f7xu + f7xw + f7xy + report_f7va_f7ud + dons_partipol
        max = P.dons_interet_general.plafond * rni

        return red_7ud_7va + P.dons_aux_oeuvres.taux * min_(base, max)

    def formula_2021_01_01(foyer_fiscal, period, parameters):
        '''
        Dons versés à d’autres organismes d’intérêt général,
        aux associations d’utilité publique, aux candidats aux élections (2021)
        '''
        rni = foyer_fiscal('rni', period)
        f7uf = foyer_fiscal('f7uf', period)
        f7uh = foyer_fiscal('f7uh', period)
        f7uj = foyer_fiscal('f7uj', period)
        f7xs = foyer_fiscal('f7xs', period)
        f7xt = foyer_fiscal('f7xt', period)
        f7xu = foyer_fiscal('f7xu', period)
        f7xw = foyer_fiscal('f7xw', period)
        f7xy = foyer_fiscal('f7xy', period)
        f7va = foyer_fiscal('f7va', period)
        f7ud = foyer_fiscal('f7ud', period)
        f7vc = foyer_fiscal('f7vc', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.dons
        plafond_reduction_don_coluche = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.plafond
        taux_donapd = parameters(period).impot_revenu.calcul_reductions_impots.dons.dons_coluche.taux
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)

        red_7ud_7va = min_(plafond_reduction_don_coluche, f7va + f7ud) * taux_donapd
        report_f7va_f7ud = max_(0, f7va + f7ud - plafond_reduction_don_coluche)

        red_7uj = min_(P.dons_assoc_cult, f7uj) * taux_donapd
        report_7uj = max_(0, f7uj - P.dons_assoc_cult)

        dons_partipol = min_(P.dons_aux_partis_politiques.plafond_seul * (1 + maries_ou_pacses), f7uh)

        base = f7uf + f7vc + f7xs + f7xt + f7xu + f7xw + f7xy + report_f7va_f7ud + report_7uj + dons_partipol
        max = P.dons_interet_general.plafond * rni

        return red_7ud_7va + red_7uj + P.dons_aux_oeuvres.taux * min_(base, max)


class reduction_enfants_scolarises(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'reduction_enfants_scolarises'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        Réduction d'impôt au titre des enfants à charge poursuivant leurs études secondaires ou supérieures
        '''
        f7ea = foyer_fiscal('f7ea', period)
        f7eb = foyer_fiscal('f7eb', period)
        f7ec = foyer_fiscal('f7ec', period)
        f7ed = foyer_fiscal('f7ed', period)
        f7ef = foyer_fiscal('f7ef', period)
        f7eg = foyer_fiscal('f7eg', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.enfants_scolarises

        return (
            P.college * (f7ea + f7eb / 2)
            + P.lycee * (f7ec + f7ed / 2)
            + P.universite * (f7ef + f7eg / 2)
            )


class interets_paiements_differes_agriculteurs(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Intérêts paiement différé agriculteurs'
    definition_period = YEAR

    def formula_2005_01_01(foyer_fiscal, period, parameters):
        '''
        Intérêts pour paiement différé accordé aux agriculteurs
        2005-
        '''
        f7um = foyer_fiscal('f7um', period)
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.interets_paiements_differes_agriculteurs

        max1 = P.plafond * (1 + maries_ou_pacses)
        return P.taux * min_(f7um, max1)


class mecena(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Mécénat d'entreprise"
    definition_period = YEAR

    def formula_2003_01_01(foyer_fiscal, period, parameters):
        '''
        Mécénat d'entreprise (case 7US)
        2003-
        '''
        f7us = foyer_fiscal('f7us', period)

        return f7us


class prestations_compensatoires(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Prestations compensatoires'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        Prestations compensatoires
        '''
        f7wm = foyer_fiscal('f7wm', period)
        f7wn = foyer_fiscal('f7wn', period)
        f7wo = foyer_fiscal('f7wo', period)
        f7wp = foyer_fiscal('f7wp', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.prestations_compensatoires

        div = (f7wo == 0) * 1 + f7wo  # Pour éviter les divisions par zéro

        return (
            (f7wm == 0) * (
                (f7wn == f7wo) * P.taux * min_(f7wn, P.plafond)
                + (f7wn < f7wo) * (f7wo <= P.plafond) * P.taux * f7wn
                + max_(0, (f7wn < f7wo) * (f7wo > P.plafond) * P.taux * P.plafond * f7wn / div)
                )
            + (f7wm != 0) * (
                (f7wn == f7wm) * (f7wo <= P.plafond) * P.taux * f7wm
                + max_(0, (f7wn == f7wm) * (f7wo >= P.plafond) * P.taux * f7wm / div)
                + (f7wn > f7wm) * (f7wo <= P.plafond) * P.taux * f7wn
                + max_(0, (f7wn > f7wm) * (f7wo >= P.plafond) * P.taux * f7wn / div)
                )
            + P.taux * f7wp
            )


class reduction_cotisations_syndicales(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Réduction d'impôt pour cotisations syndicales"
    definition_period = YEAR
    end = '2011-12-31'

    def formula(foyer_fiscal, period, parameters):
        '''
        Cotisations syndicales : réduction d'impôt (2002-2011) puis crédit d'impôt (2012- )
        '''
        return foyer_fiscal('cotisations_syndicales', period)


class cotisations_syndicales(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Montant de la réduction ou crédit d'impôt pour cotisations syndicales"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        NB: This is but an approximation of the true calculation, as it is done on the level of the tax unit.
        In reality, the ceilings of 1 % of revenues are applied on the individual level (cf. BOI-IR-RICI-20).
        '''

        f7ac = foyer_fiscal.members('f7ac', period)

        cotisations_versees = f7ac

        salaire_imposable = foyer_fiscal.members('salaire_imposable', period, options = [ADD])
        chomage_imposable = foyer_fiscal.members('chomage_imposable', period, options = [ADD])
        retraite_imposable = foyer_fiscal.members('retraite_imposable', period, options = [ADD])

        P = parameters(period).impot_revenu.calcul_reductions_impots.cotisations_syndicales

        plafond = (salaire_imposable + chomage_imposable + retraite_imposable) * P.plafond

        return (P.taux * foyer_fiscal.sum(min_(cotisations_versees, plafond)))


class interets_emprunt_reprise_societe(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Intérêts emprunts pour reprise de société'
    definition_period = YEAR

    def formula_2003_01_01(foyer_fiscal, period, parameters):
        '''
        Intérêts d'emprunts pour reprise de société
        2003-
        '''
        maries_ou_pacses = foyer_fiscal('maries_ou_pacses', period)
        f7fh = foyer_fiscal('f7fh', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.interets_emprunt_reprise_societe

        plafond = P.plafond * (maries_ou_pacses + 1)
        return P.taux * min_(f7fh, plafond)


class restauration_patrimoine_bati(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Réduction d'impôt au titre des dépenses de restauration immobilière - Dispositif Malraux"
    reference = 'http://bofip.impots.gouv.fr/bofip/1372-PGP'
    definition_period = YEAR

    def formula_2009_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière (cases 7RA et 7RB)
        2009-2010
        '''
        f7ra = foyer_fiscal('f7ra_2015', period)
        f7rb = foyer_fiscal('f7rb_2015', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        max1 = P.plafond
        max2 = max_(max1 - f7rb, 0)
        return P.taux_rb * min_(f7rb, max1) + P.taux_ra * min_(f7ra, max2)

    def formula_2011_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière (cases 7RA, 7RB, 7RC, 7RD)
        2011
        '''
        f7ra = foyer_fiscal('f7ra_2015', period)
        f7rb = foyer_fiscal('f7rb_2015', period)
        f7rc = foyer_fiscal('f7rc_2015', period)
        f7rd = foyer_fiscal('f7rd_2015', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        max1 = P.plafond
        max2 = max_(max1 - f7rd, 0)
        max3 = max_(max2 - f7rb, 0)
        max4 = max_(max3 - f7rc, 0)

        return (
            P.taux_rd * min_(f7rd, max1)
            + P.taux_rb * min_(f7rb, max2)
            + P.taux_rc * min_(f7rc, max3)
            + P.taux_ra * min_(f7ra, max4)
            )

    def formula_2012_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière (cases 7RA, 7RB, 7RC, 7RD, 7RE, 7RF)
        2012
        '''
        f7ra = foyer_fiscal('f7ra_2015', period)
        f7rb = foyer_fiscal('f7rb_2015', period)
        f7rc = foyer_fiscal('f7rc_2015', period)
        f7rd = foyer_fiscal('f7rd_2015', period)
        f7re = foyer_fiscal('f7re_2016', period)
        f7rf = foyer_fiscal('f7rf_2016', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        max1 = P.plafond
        max2 = max_(max1 - f7rd, 0)
        max3 = max_(max2 - f7rb, 0)
        max4 = max_(max3 - f7rc - f7rf, 0)
        max5 = max_(max4 - f7ra, 0)

        return (
            P.taux_rd * min_(f7rd, max1)
            + P.taux_rb * min_(f7rb, max2)
            + P.taux_rc * min_(f7rc + f7rf, max3)
            + P.taux_ra * min_(f7ra, max4)
            + P.taux_re * min_(f7re, max5)
            )

    def formula_2013_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière (cases 7RA, 7RB, 7RC, 7RD, 7RE, 7RF, 7SX, 7SY)
        2013-2015
        '''
        f7ra = foyer_fiscal('f7ra_2015', period)
        f7rb = foyer_fiscal('f7rb_2015', period)
        f7rc = foyer_fiscal('f7rc_2015', period)
        f7rd = foyer_fiscal('f7rd_2015', period)
        f7re = foyer_fiscal('f7re_2016', period)
        f7rf = foyer_fiscal('f7rf_2016', period)
        f7sx = foyer_fiscal('f7sx_2017', period)
        f7sy = foyer_fiscal('f7sy_2017', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        max1 = P.plafond
        max2 = max_(max1 - f7rd, 0)
        max3 = max_(max2 - f7rb, 0)
        max4 = max_(max3 - f7rc - f7sy - f7rf, 0)
        max5 = max_(max4 - f7ra, 0)

        return (
            P.taux_rd * min_(f7rd, max1)
            + P.taux_rb * min_(f7rb, max2)
            + P.taux_rc * min_(f7sy + f7rf + f7rc, max3)
            + P.taux_ra * min_(f7ra, max4) + P.taux_re * min_(f7re + f7sx, max5)
            )

    def formula_2016_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière
        2016
        '''
        f7nx = foyer_fiscal('f7nx', period)
        f7ny = foyer_fiscal('f7ny', period)
        f7re = foyer_fiscal('f7re_2016', period)
        f7rf = foyer_fiscal('f7rf_2016', period)
        f7sx = foyer_fiscal('f7sx_2017', period)
        f7sy = foyer_fiscal('f7sy_2017', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        max1 = P.plafond
        max2 = max_(max1 - f7nx - f7sy - f7rf, 0)

        return (
            P.taux_rc * min_(f7sy + f7rf + f7nx, max1)
            + P.taux_re * min_(f7re + f7sx + f7ny, max2)
            )

    def formula_2017_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière
        2017
        '''
        f7sy = foyer_fiscal('f7sy_2017', period)
        f7sx = foyer_fiscal('f7sx_2017', period)

        f7nx = foyer_fiscal('f7nx', period)
        f7ny = foyer_fiscal('f7ny', period)

        f7tx = foyer_fiscal('f7tx', period)
        f7ty = foyer_fiscal('f7ty', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati
        P0 = parameters('2009-01-01').impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        depenses_secteur_degrade = f7sy + f7nx
        depenses_secteur_patrimoine_remarquable = f7sx + f7ny
        depenses_PSMV_2017 = f7tx
        depenses_non_PSMV_2017 = f7ty

        max1 = max_(P0.plafond - depenses_secteur_degrade, 0)
        max3 = max_(P.plafond - depenses_PSMV_2017, 0)

        return (
            P.taux_30 * (
                min_(depenses_secteur_degrade, P0.plafond)
                + min_(depenses_PSMV_2017, P.plafond)
                )
            + P.taux_22 * (
                min_(depenses_secteur_patrimoine_remarquable, max1)
                + min_(depenses_non_PSMV_2017, max3)
                )
            )

    def formula_2018_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière
        2018
        '''
        # plaf 100K
        f7nx = foyer_fiscal('f7nx', period)
        f7ny = foyer_fiscal('f7ny', period)

        # plaf 400K
        f7tx = foyer_fiscal('f7tx', period)
        f7ty = foyer_fiscal('f7ty', period)

        # reports
        f7kz = foyer_fiscal('f7kz', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati
        P0 = parameters('2009-01-01').impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        base_nx = min_(P.plafond, f7nx)
        base_ny = min_(P.plafond - f7nx, f7ny)

        base_tx = min_(P0.plafond, f7tx)
        base_ty = min_(P0.plafond - f7tx, f7ty)

        ri = (f7kz
            + P.taux_30 * (base_nx + base_tx)
            + P.taux_22 * (base_ny + base_ty))

        return ri

    def formula_2019_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière
        2019
        '''
        # plaf 100K
        f7nx = foyer_fiscal('f7nx', period)
        f7ny = foyer_fiscal('f7ny', period)

        # plaf 400K
        f7tx = foyer_fiscal('f7tx', period)
        f7ty = foyer_fiscal('f7ty', period)

        # reports
        f7kz = foyer_fiscal('f7kz', period)
        f7ky = foyer_fiscal('f7ky', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati
        P0 = parameters('2019-01-01').impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        base_nx = min_(P.plafond, f7nx)
        base_ny = min_(P.plafond - f7nx, f7ny)

        base_tx = min_(P0.plafond, f7tx)
        base_ty = min_(P0.plafond - f7tx, f7ty)

        ri = (f7kz + f7ky
            + P.taux_30 * (base_nx + base_tx)
            + P.taux_22 * (base_ny + base_ty))

        return ri

    def formula_2020_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière
        2020
        '''
        # plaf 100K
        f7nx = foyer_fiscal('f7nx', period)
        f7ny = foyer_fiscal('f7ny', period)

        # plaf 400K
        f7tx = foyer_fiscal('f7tx', period)
        f7ty = foyer_fiscal('f7ty', period)

        # reports
        f7kz = foyer_fiscal('f7kz', period)
        f7ky = foyer_fiscal('f7ky', period)
        f7kx = foyer_fiscal('f7kx', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati
        P0 = parameters('2019-01-01').impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        base_nx = min_(P.plafond, f7nx)
        base_ny = min_(P.plafond - f7nx, f7ny)

        base_tx = min_(P0.plafond, f7tx)
        base_ty = min_(P0.plafond - f7tx, f7ty)

        ri = (f7kz + f7ky + f7kx
            + P.taux_30 * (base_nx + base_tx)
            + P.taux_22 * (base_ny + base_ty))

        return ri

    def formula_2021_01_01(foyer_fiscal, period, parameters):
        '''
        Travaux de restauration immobilière
        2021
        '''
        # plaf 400K
        f7tx = foyer_fiscal('f7tx', period)
        f7ty = foyer_fiscal('f7ty', period)

        # reports
        f7ky = foyer_fiscal('f7ky', period)
        f7kx = foyer_fiscal('f7kx', period)
        f7kw = foyer_fiscal('f7kw', period)

        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.restauration_patrimoine_bati

        base_tx = min_(P.plafond, f7tx)
        base_ty = min_(P.plafond - f7tx, f7ty)

        ri = (f7ky + f7kx + f7kw
            + P.taux_30 * (base_tx)
            + P.taux_22 * (base_ty))

        return ri


class rente_survie(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'rente_survie'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        Rentes de survie et contrats d'épargne handicap
        2002-
        '''
        nb_pac_majoration_plafond = foyer_fiscal('nb_pac2', period)
        nbR = foyer_fiscal('nbR', period)
        f7gz = foyer_fiscal('f7gz', period)
        P = parameters(period).impot_revenu.calcul_reductions_impots.divers.rente_survie

        max1 = P.plafond + (nb_pac_majoration_plafond - nbR) * P.increment
        return P.taux * min_(f7gz, max1)



from openfisca_france.model.base import *


# Simulation TH de la résidence principale : législation à partir de l'année 2017


class valeur_locative_cadastrale_brute(Variable):
    value_type = float
    entity = Menage
    label = 'Valeur locative cadastrale utilisée pour les impôts locaux, avant abattements'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?idArticle=LEGIARTI000034111395&cidTexte=LEGITEXT000006069577&dateTexte=20170302'
    definition_period = YEAR


class condition_rfr_exoneration_th(Variable):
    value_type = bool
    default_value = False
    entity = FoyerFiscal
    label = "Condition de revenu fiscal de référence pour l'éxonération à l'échelle du foyer fiscal"
    reference = 'http://bofip.impots.gouv.fr/bofip/5934-PGP.html'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(foyer_fiscal, period, parameters):
        '''
        Pour l'exonération de la taxe d'habitation et l'abattement pour condition modeste, en cas de ménages à foyers
        fiscaux multiples, la condition relative au revenu fiscal de référence doit être respectée pour tous les foyers
        fiscaux du ménage, d'où cette variable intermédiaire
        '''
        P = parameters(period).taxation_indirecte.taxe_habitation
        rfr = foyer_fiscal('rfr', period.last_year)
        nbptr = foyer_fiscal('nbptr', period.last_year)
        seuil_th = P.exon_plaf_rfr.premiere_part + P.exon_plaf_rfr.demi_part_supp * max_(0, (nbptr - 1) * 2)
        return (rfr < seuil_th)


class exonere_taxe_habitation(Variable):
    value_type = bool
    default_value = False
    entity = Menage
    label = "Exonération de la taxe d'habitation"
    reference = 'http://vosdroits.service-public.fr/particuliers/F42.xhtml'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Hypothèses :
            (1) pour la condition de plus de 60 ans ou veuf, on regarde seulement la personne de référence du ménage
            (2) pour la condition relative à l'ASPA l'ASI et l'AAH, on fait la somme de ces prestations à l'échelle du ménage
            (3) pour la condition relative à l'ISF-IFI, on fait la somme de ces impôts à l'échelle du ménage
            (4) on ne prend pas en compte les prolongements temporaires d'exonération pour certains contribuables
                ayant perdu l'éligibilité à l'éxonération (I bis de l'art. 1414 du CGI)
        '''
        janvier = period.first_month

        age_personne_de_reference = menage.personne_de_reference('age', janvier)
        age_conjoint = menage.conjoint('age', janvier)
        statut_marital = menage.personne_de_reference('statut_marital', janvier)

        aah_i = menage.members('aah', period, options = [ADD])
        asi_i = menage.members('asi', period, options = [ADD])
        aspa_i = menage.members.famille('aspa', period, options = [ADD])
        aah = menage.sum(aah_i)
        asi = menage.sum(asi_i)
        aspa = menage.sum(aspa_i, role = Famille.DEMANDEUR)

        isf_ifi_i = menage.members.foyer_fiscal('isf_ifi', period.last_year)
        isf_ifi = menage.sum(isf_ifi_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        condition_rfr_exoneration_th_i = menage.members.foyer_fiscal('condition_rfr_exoneration_th', period)
        condition_rfr_exoneration_th = menage.all(condition_rfr_exoneration_th_i)

        P = parameters(period).taxation_indirecte.taxe_habitation

        exon_non_soumis_a_condition_rfr = (asi > 0) + (aspa > 0)
        exon_soumis_a_condition_rfr = ((age_personne_de_reference >= P.exon_age_min) + (age_conjoint >= P.exon_age_min) + (statut_marital == TypesStatutMarital.veuf)) * (isf_ifi == 0) + (aah > 0)
        exon = exon_non_soumis_a_condition_rfr + exon_soumis_a_condition_rfr * condition_rfr_exoneration_th
        return exon


class abattement_charge_famille_th_commune(Variable):
    value_type = float
    entity = Menage
    label = 'Abattement obligatoire pour charges de famille - TH de la commune'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006069577&idArticle=LEGIARTI000033220348&dateTexte=&categorieLien=id'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Dans les personnes à charge :
            (1) on ne compte pas les ascendants de plus de 70 ans ou infirmes, ayant un
                revenu fiscal de référence inférieur à un certain seuil.
            (2) on ne prend pas en compte les gardes en résidence alternée qui font qu'une personne à charge ne compte
                que pour 0.5 au lieu de 1.
        '''
        enfant_i = menage.members.has_role(Famille.ENFANT)
        nb_enfants = menage.sum(enfant_i)
        abt_pac_1_2_th_commune = menage('abt_pac_1_2_th_commune', period)
        abt_pac_3pl_th_commune = menage('abt_pac_3pl_th_commune', period)
        return (
            abt_pac_1_2_th_commune * min_(nb_enfants, 2)
            + abt_pac_3pl_th_commune * max_(nb_enfants - 2, 0)
            )


class abattement_charge_famille_th_epci(Variable):
    value_type = float
    entity = Menage
    label = "Abattement obligatoire pour charges de famille - TH de l'EPCI"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006069577&idArticle=LEGIARTI000033220348&dateTexte=&categorieLien=id'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Dans les personnes à charge :
            (1) on ne compte pas les ascendants de plus de 70 ans ou infirmes, ayant un
                revenu fiscal de référence inférieur à un certain seuil.
            (2) on ne prend pas en compte les gardes en résidence alternée qui font qu'une personne à charge ne compte
                que pour 0.5 au lieu de 1.
        '''
        enfant_i = menage.members.has_role(Famille.ENFANT)
        nb_enfants = menage.sum(enfant_i)
        abt_pac_1_2_th_epci = menage('abt_pac_1_2_th_epci', period)
        abt_pac_3pl_th_epci = menage('abt_pac_3pl_th_epci', period)
        return (
            abt_pac_1_2_th_epci * min_(nb_enfants, 2)
            + abt_pac_3pl_th_epci * max_(nb_enfants - 2, 0)
            )


class abattement_personnes_condition_modeste_th_commune(Variable):
    value_type = float
    entity = Menage
    label = 'Abattement pour personnes de condition modeste - TH de la commune'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006069577&idArticle=LEGIARTI000033220348&dateTexte=&categorieLien=id'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Pour le nombre de personnes à charge, on ne prend pas en compte les gardes en résidence alternée qui font
        qu'une personne à charge ne compte que pour 0.5 au lieu de 1.
        '''
        enfant_i = menage.members.has_role(Famille.ENFANT)
        nb_enfants = menage.sum(enfant_i)
        valeur_locative_cadastrale_brute = menage('valeur_locative_cadastrale_brute', period)
        P = parameters(period).taxation_indirecte.taxe_habitation
        abt_condition_modeste_th_commune = menage('abt_condition_modeste_th_commune', period)
        valeur_locative_moyenne_th_commune = menage('valeur_locative_moyenne_th_commune', period)
        taux_plafond_general = P.seuil_valeur_locative_abattement_condition_modeste
        maj_taux_plafond_par_pac = P.maj_seuil_valeur_locative_abattement_condition_modeste
        valeur_locative_max = (taux_plafond_general + maj_taux_plafond_par_pac * nb_enfants) * valeur_locative_moyenne_th_commune
        condition_rfr_exoneration_th_i = menage.members.foyer_fiscal('condition_rfr_exoneration_th', period)
        condition_rfr_exoneration_th = menage.all(condition_rfr_exoneration_th_i)
        exonere_taxe_habitation = menage('exonere_taxe_habitation', period)
        elig = condition_rfr_exoneration_th * not_(exonere_taxe_habitation) * (valeur_locative_cadastrale_brute <= valeur_locative_max)

        return elig * abt_condition_modeste_th_commune


class abattement_personnes_condition_modeste_th_epci(Variable):
    value_type = float
    entity = Menage
    label = "Abattement pour personnes de condition modeste - TH de l'EPCI"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006069577&idArticle=LEGIARTI000033220348&dateTexte=&categorieLien=id'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Pour le nombre de personnes à charge, on ne prend pas en compte les gardes en résidence alternée qui font
        qu'une personne à charge ne compte que pour 0.5 au lieu de 1.
        '''
        enfant_i = menage.members.has_role(Famille.ENFANT)
        nb_enfants = menage.sum(enfant_i)
        valeur_locative_cadastrale_brute = menage('valeur_locative_cadastrale_brute', period)
        P = parameters(period).taxation_indirecte.taxe_habitation
        abt_condition_modeste_th_epci = menage('abt_condition_modeste_th_epci', period)
        valeur_locative_moyenne_th_epci = menage('valeur_locative_moyenne_th_epci', period)
        taux_plafond_general = P.seuil_valeur_locative_abattement_condition_modeste
        maj_taux_plafond_par_pac = P.maj_seuil_valeur_locative_abattement_condition_modeste
        valeur_locative_max = (taux_plafond_general + maj_taux_plafond_par_pac * nb_enfants) * valeur_locative_moyenne_th_epci
        condition_rfr_exoneration_th_i = menage.members.foyer_fiscal('condition_rfr_exoneration_th', period)
        condition_rfr_exoneration_th = menage.all(condition_rfr_exoneration_th_i)
        exonere_taxe_habitation = menage('exonere_taxe_habitation', period)
        elig = condition_rfr_exoneration_th * not_(exonere_taxe_habitation) * (valeur_locative_cadastrale_brute <= valeur_locative_max)

        return elig * abt_condition_modeste_th_epci


class base_nette_th_commune(Variable):
    value_type = float
    entity = Menage
    label = 'Base nette - TH de la commune'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Notes :
            (1) on ne prend pas en compte l'abattement en faveur des personnes handicapées
            (2) on ne prend pas en compte les abattements temporaires pour certains contribuables
                ayant perdu l'éligibilité à l'éxonération (I bis de l'art. 1414 du CGI)
        '''
        valeur_locative_cadastrale_brute = menage('valeur_locative_cadastrale_brute', period)
        abattement_charge_famille_th_commune = menage('abattement_charge_famille_th_commune', period)
        abattement_personnes_condition_modeste_th_commune = menage('abattement_personnes_condition_modeste_th_commune', period)
        abt_general_base_th_commune = menage('abt_general_base_th_commune', period)
        base_brute_moins_abattements = (
            valeur_locative_cadastrale_brute
            - abattement_charge_famille_th_commune
            - abattement_personnes_condition_modeste_th_commune
            - abt_general_base_th_commune
            )
        return max_(base_brute_moins_abattements, 0)


class base_nette_th_epci(Variable):
    value_type = float
    entity = Menage
    label = "Base nette - TH de l'EPCI"
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Notes :
            (1) on ne prend pas en compte l'abattement en faveur des personnes handicapées
            (2) on ne prend pas en compte les abattements temporaires pour certains contribuables
                ayant perdu l'éligibilité à l'éxonération (I bis de l'art. 1414 du CGI)
        '''
        valeur_locative_cadastrale_brute = menage('valeur_locative_cadastrale_brute', period)
        abattement_charge_famille_th_epci = menage('abattement_charge_famille_th_epci', period)
        abattement_personnes_condition_modeste_th_epci = menage('abattement_personnes_condition_modeste_th_epci', period)
        abt_general_base_th_epci = menage('abt_general_base_th_epci', period)
        base_brute_moins_abattements = (
            valeur_locative_cadastrale_brute
            - abattement_charge_famille_th_epci
            - abattement_personnes_condition_modeste_th_epci
            - abt_general_base_th_epci
            )
        return max_(base_brute_moins_abattements, 0)


class taxe_habitation_commune_epci_avant_degrevement(Variable):
    value_type = float
    entity = Menage
    label = "Taxe d'habitation de la commune et de l'EPCI avant dégrèvement (frais de gestion inclus)"
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period, parameters):
        taux_th_commune = menage('taux_th_commune', period)
        taux_th_epci = menage('taux_th_epci', period)
        # Attention, si (taux_th_commune + taux_th_epci == 0), alors
        # il n'y a pas de taux de taxe d'habitation défini pour le depcom indiqué.

        base_nette_th_commune = menage('base_nette_th_commune', period)
        base_nette_th_epci = menage('base_nette_th_epci', period)
        exonere_taxe_habitation = menage('exonere_taxe_habitation', period)
        P = parameters(period).taxation_indirecte.taxe_habitation
        taux_frais_assiette = P.frais_assiette
        return (base_nette_th_commune * taux_th_commune + base_nette_th_epci * taux_th_epci) * not_(exonere_taxe_habitation) * (1 + taux_frais_assiette)


class plafond_taxe_habitation_eligibilite(Variable):
    value_type = bool
    entity = Menage
    label = "Eligibilité au plafond de la taxe d'habitation en fonction du revenu fiscal de référence"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006069577&idArticle=LEGIARTI000006312070&dateTexte=&categorieLien=cid'
    definition_period = YEAR
    end = '2019-12-31'

    def formula_2017_01_01(menage, period, parameters):
        P_plaf = parameters(period).taxation_indirecte.taxe_habitation.plafonnement
        rfr_i = menage.members.foyer_fiscal('rfr', period.last_year)
        rfr_menage = menage.sum(rfr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        nbptr_i = menage.members.foyer_fiscal('nbptr', period.last_year)
        nbptr_menage = menage.sum(nbptr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        isf_ifi_i = menage.members.foyer_fiscal('isf_ifi', period.last_year)
        isf_ifi_menage = menage.sum(isf_ifi_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        seuil_rfr = P_plaf.plaf_rfr.premiere_part + P_plaf.plaf_rfr.premiere_demi_part_supp * (min_(max_(nbptr_menage - 1, 0), 0.5)) / 0.5 + P_plaf.plaf_rfr.autres_demi_parts_supp * (max_(nbptr_menage - 1.5, 0)) / 0.5
        return (rfr_menage <= seuil_rfr) * (isf_ifi_menage == 0)


class plafond_taxe_habitation(Variable):
    value_type = float
    entity = Menage
    label = "Plafond de la taxe d'habitation en fonction du revenu fiscal de référence"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006069577&idArticle=LEGIARTI000006312070&dateTexte=&categorieLien=cid'
    definition_period = YEAR
    end = '2019-12-31'

    def formula_2017_01_01(menage, period, parameters):
        plafond_taxe_habitation_eligibilite = menage('plafond_taxe_habitation_eligibilite', period)
        P_plaf = parameters(period).taxation_indirecte.taxe_habitation.plafonnement
        rfr_i = menage.members.foyer_fiscal('rfr', period.last_year)
        rfr_menage = menage.sum(rfr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        nbptr_i = menage.members.foyer_fiscal('nbptr', period.last_year)
        nbptr_menage = menage.sum(nbptr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        abattement = P_plaf.abattement_rfr.premiere_part + P_plaf.abattement_rfr.quatre_premieres_demi_parts_supp * (min_(max_(nbptr_menage - 1, 0), 2)) / 0.5 + P_plaf.abattement_rfr.autres_demi_parts_supp * (max_(nbptr_menage - 3, 0)) / 0.5
        return max_(rfr_menage - abattement, 0) * P_plaf.taux_plafonnement_revenu * plafond_taxe_habitation_eligibilite


class degrevement_plafonnement_taxe_habitation(Variable):
    value_type = float
    entity = Menage
    label = "Dégrèvement de la taxe d'habitation au titre du plafonnement"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006069577&idArticle=LEGIARTI000006312070&dateTexte=&categorieLien=cid'
    definition_period = YEAR
    end = '2019-12-31'

    def formula_2017_01_01(menage, period, parameters):
        '''
        Notes :
            (1) On ne prend pas en compte la majoration du dégrèvement pour les cas spécifiques où le montant
                de taxe résulte exclusivement de la réduction du dégrèvement dépendant des hausses de taux
                depuis 2000 (3. du III. de l'art 1414 A du CGI)
            (2) Pour la réduction du dégrèvement, on ne prend pas en compte les cas où les abattements du
                département en 2010 étaient plus avantageux que les abattements de la commune ou de l'EPCI
                en 2018 (a. du 1. du III. de l'art 1414 A du CGI)
        '''
        taxe_habitation_commune_epci_avant_degrevement = menage('taxe_habitation_commune_epci_avant_degrevement', period)
        plafond_taxe_habitation_eligibilite = menage('plafond_taxe_habitation_eligibilite', period)
        plafond_taxe_habitation = menage('plafond_taxe_habitation', period)
        base_nette_th_commune = menage('base_nette_th_commune', period)
        base_nette_th_epci = menage('base_nette_th_epci', period)
        base_reduction_degrevement = min_(base_nette_th_commune, base_nette_th_epci)

        taux_th_commune = menage('taux_th_commune', period)
        taux_th_epci = menage('taux_th_epci', period)
        ecart_avec_2000 = period.start.offset('first-of', 'year').year - 2000
        annee_2000 = period.start.offset('first-of', 'year').period('year').offset(-ecart_avec_2000)
        taux_th_commune_2000 = menage('taux_th_commune', annee_2000)
        taux_th_epci_2000 = menage('taux_th_epci', annee_2000)

        P = parameters(period).taxation_indirecte.taxe_habitation
        reduction_degrevement = base_reduction_degrevement * (taux_th_commune + taux_th_epci - (taux_th_commune_2000 + taux_th_epci_2000) * P.plafonnement.coeff_multiplicateur_taux_2000)
        reduction_degrevement = reduction_degrevement * (reduction_degrevement > P.plafonnement.valeur_minimale_reduction_degrevement)

        # Les taux de taxe d'habitation de l'année 2000 utilisés pour la réduction du plafonnement
        # ne sont pas disponibles pour cette commune. Nous mettons donc cette réduction à zéro.
        reduction_degrevement = where(
            taux_th_commune_2000 + taux_th_epci_2000 == 0,
            0,
            reduction_degrevement,
            )

        degrevement = (
            taxe_habitation_commune_epci_avant_degrevement
            - plafond_taxe_habitation
            - reduction_degrevement
            )
        return max_(degrevement, 0) * plafond_taxe_habitation_eligibilite


class taxe_habitation_commune_epci_apres_degrevement_plafonnement(Variable):
    value_type = float
    entity = Menage
    label = "Taxe d'habitation de la commune et de l'EPCI après dégrèvement pour plafonnement"
    definition_period = YEAR
    end = '2019-12-31'

    def formula_2017_01_01(menage, period, parameters):
        taxe_habitation_commune_epci_avant_degrevement = menage('taxe_habitation_commune_epci_avant_degrevement', period)
        degrevement_plafonnement_taxe_habitation = menage('degrevement_plafonnement_taxe_habitation', period)
        return max_(taxe_habitation_commune_epci_avant_degrevement - degrevement_plafonnement_taxe_habitation, 0)


class degrevement_office_taxe_habitation(Variable):
    value_type = float
    entity = Menage
    label = "Dégrèvement d'office de la taxe d'habitation"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=268AD735F5C69F94FB6756E3125BF0A0.tplgfr43s_1?idArticle=LEGIARTI000036441815&cidTexte=LEGITEXT000006069577&dateTexte=20190513&categorieLien=id&oldAction=&nbResultRech='
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2018_01_01(menage, period, parameters):
        '''
        Note Importante : on ne prend pas en compte l'exclusion de ce dégrèvement des hausses de taux et d'abattements depuis 2018.
        Dans les faits :
            - si les variations de taux de taxation et d'abattements votés par les communes et EPCI depuis 2017
              entrainent une baisse ou une non variation de la taxe d'habitation, le dégrèvement s'applique au
              montant de taxe d'habitation avec taux et abattements actualisés;
            - si ces variations entrainent une hausse de la taxe d'habitation, le dégrèvement d'office est
              appliqué à la taxe d'habitation qui aurait été payée en N (avec N>=2018) avec les taux de taxation
              et d'abattements votés par les communes et EPCI en 2017.
        Dans ce code, on suppose qu'on est toujours dans le premier cas.
        '''

        # Calcul de l'éligibilité en fonction du revenu fiscal de référence et de la perception de l'ISF-IFI
        P_degrev = parameters(period).taxation_indirecte.taxe_habitation.degrevement_d_office
        isf_ifi_i = menage.members.foyer_fiscal('isf_ifi', period.last_year)
        isf_ifi_menage = menage.sum(isf_ifi_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        rfr_i = menage.members.foyer_fiscal('rfr', period.last_year)
        rfr_menage = menage.sum(rfr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        nbptr_i = menage.members.foyer_fiscal('nbptr', period.last_year)
        nbptr_menage = menage.sum(nbptr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        plafond_rfr_degrev = P_degrev.plaf_rfr_degrev.premiere_part + P_degrev.plaf_rfr_degrev.deux_premieres_demi_parts_supp * (min_(max_(nbptr_menage - 1, 0), 1)) / 0.5 + P_degrev.plaf_rfr_degrev.autres_demi_parts_supp * (max_(nbptr_menage - 2, 0)) / 0.5
        plafond_rfr_degrev_degressif = P_degrev.plaf_rfr_degrev_degressif.premiere_part + P_degrev.plaf_rfr_degrev_degressif.deux_premieres_demi_parts_supp * (min_(max_(nbptr_menage - 1, 0), 1)) / 0.5 + P_degrev.plaf_rfr_degrev_degressif.autres_demi_parts_supp * (max_(nbptr_menage - 2, 0)) / 0.5
        elig_degrev = (isf_ifi_menage == 0) * (rfr_menage <= plafond_rfr_degrev)
        elig_degrev_degressif = (isf_ifi_menage == 0) * (elig_degrev == 0) * (rfr_menage <= plafond_rfr_degrev_degressif)

        # Calcul du dégrèvement
        taxe_habitation_commune_epci_apres_degrevement_plafonnement = menage('taxe_habitation_commune_epci_apres_degrevement_plafonnement', period)
        degrev = P_degrev.taux_inf_plaf * taxe_habitation_commune_epci_apres_degrevement_plafonnement
        degrev_degressif = degrev * max_((plafond_rfr_degrev_degressif - rfr_menage) / (plafond_rfr_degrev_degressif - plafond_rfr_degrev), 0)

        return degrev * elig_degrev + degrev_degressif * elig_degrev_degressif

    def formula_2020_01_01(menage, period, parameters):
        '''
        Différence par rapport à la formule précédente : le taux de plafonnement est appliqué à taxe_habitation_commune_epci_avant_degrevement
        (au lieu de taxe_habitation_commune_epci_apres_degrevement_plafonnement)
        '''

        # Calcul de l'éligibilité en fonction du revenu fiscal de référence et de la perception de l'ISF-IFI
        P_degrev = parameters(period).taxation_indirecte.taxe_habitation.degrevement_d_office
        isf_ifi_i = menage.members.foyer_fiscal('isf_ifi', period.last_year)
        isf_ifi_menage = menage.sum(isf_ifi_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        rfr_i = menage.members.foyer_fiscal('rfr', period.last_year)
        rfr_menage = menage.sum(rfr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        nbptr_i = menage.members.foyer_fiscal('nbptr', period.last_year)
        nbptr_menage = menage.sum(nbptr_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        plafond_rfr_degrev = P_degrev.plaf_rfr_degrev.premiere_part + P_degrev.plaf_rfr_degrev.deux_premieres_demi_parts_supp * (min_(max_(nbptr_menage - 1, 0), 1)) / 0.5 + P_degrev.plaf_rfr_degrev.autres_demi_parts_supp * (max_(nbptr_menage - 2, 0)) / 0.5
        plafond_rfr_degrev_degressif = P_degrev.plaf_rfr_degrev_degressif.premiere_part + P_degrev.plaf_rfr_degrev_degressif.deux_premieres_demi_parts_supp * (min_(max_(nbptr_menage - 1, 0), 1)) / 0.5 + P_degrev.plaf_rfr_degrev_degressif.autres_demi_parts_supp * (max_(nbptr_menage - 2, 0)) / 0.5
        elig_degrev = (isf_ifi_menage == 0) * (rfr_menage <= plafond_rfr_degrev)
        elig_degrev_degressif = (isf_ifi_menage == 0) * (elig_degrev == 0) * (rfr_menage <= plafond_rfr_degrev_degressif)
        elig_degrev_sup_plaf = (elig_degrev == 0)

        # Calcul du dégrèvement
        taxe_habitation_commune_epci_avant_degrevement = menage('taxe_habitation_commune_epci_avant_degrevement', period)
        degrev = P_degrev.taux_inf_plaf * taxe_habitation_commune_epci_avant_degrevement
        degrev_degressif = degrev * max_((plafond_rfr_degrev_degressif - rfr_menage) / (plafond_rfr_degrev_degressif - plafond_rfr_degrev), 0)
        degrev_sup_plaf = P_degrev.taux_sup_plaf * max_(taxe_habitation_commune_epci_avant_degrevement - degrev_degressif, 0)

        return degrev * elig_degrev + degrev_degressif * elig_degrev_degressif + degrev_sup_plaf * elig_degrev_sup_plaf


class prelevement_base_imposition_elevee_taxe_habitation(Variable):
    value_type = float
    entity = Menage
    label = "Prélevement sur base d'imposition élevée sur l'habitation principale"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=81889AAB5AC742D99144BC27CE3D7C31.tplgfr32s_3?idArticle=LEGIARTI000036443853&cidTexte=LEGITEXT000006069577&categorieLien=id&dateTexte=20171231'
    definition_period = YEAR
    end = '2021-12-31'

    def formula_2017_01_01(menage, period, parameters):

        P = parameters(period).taxation_indirecte.taxe_habitation.prelevement_base_imposition_elevee
        exonere_taxe_habitation = menage('exonere_taxe_habitation', period)
        degrevement_plafonnement_taxe_habitation = menage('degrevement_plafonnement_taxe_habitation', period)
        degrevement_office_taxe_habitation = menage('degrevement_office_taxe_habitation', period)
        base_nette_th_commune = menage('base_nette_th_commune', period)

        non_exonere_non_degreve = not_(exonere_taxe_habitation) * (degrevement_plafonnement_taxe_habitation == 0) * (degrevement_office_taxe_habitation == 0)
        condition_base_nette = (base_nette_th_commune > P.seuil_base_nette_habitation_principale)

        return non_exonere_non_degreve * condition_base_nette * base_nette_th_commune * P.taux_habitation_principale


class taxe_habitation(Variable):
    value_type = float
    entity = Menage
    label = "Taxe d'habitation de la commune et de l'EPCI (Établissement Public de Coopération Intercommunale), frais de gestion inclus"
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F42'
    definition_period = YEAR
    end = '2022-12-31'

    def formula_2017_01_01(menage, period):
        taxe_habitation_commune_epci_apres_degrevement_plafonnement = menage('taxe_habitation_commune_epci_apres_degrevement_plafonnement', period)
        degrevement_office_taxe_habitation = menage('degrevement_office_taxe_habitation', period)
        prelevement_base_imposition_elevee_taxe_habitation = menage('prelevement_base_imposition_elevee_taxe_habitation', period)
        return - max_(taxe_habitation_commune_epci_apres_degrevement_plafonnement - degrevement_office_taxe_habitation, 0) - prelevement_base_imposition_elevee_taxe_habitation

    def formula_2020_01_01(menage, period):
        taxe_habitation_commune_epci_avant_degrevement = menage('taxe_habitation_commune_epci_avant_degrevement', period)
        degrevement_office_taxe_habitation = menage('degrevement_office_taxe_habitation', period)
        prelevement_base_imposition_elevee_taxe_habitation = menage('prelevement_base_imposition_elevee_taxe_habitation', period)
        return - max_(taxe_habitation_commune_epci_avant_degrevement - degrevement_office_taxe_habitation, 0) - prelevement_base_imposition_elevee_taxe_habitation

    def formula_2021_01_01(menage, period):
        taxe_habitation_commune_epci_avant_degrevement = menage('taxe_habitation_commune_epci_avant_degrevement', period)
        degrevement_office_taxe_habitation = menage('degrevement_office_taxe_habitation', period)
        return - max_(taxe_habitation_commune_epci_avant_degrevement - degrevement_office_taxe_habitation, 0)
